/***************************************************************************
 *
 * Copyright (c) 2002 Promise Technology, Inc.  All rights reserved.
 * No part of this document may be reproduced or transmitted in any form or
 * by any means, electronic or mechanical, for any purpose, without the
 * express written permission of Promise Technology, Inc.
 *
 *
 * cam_def.h - defines data structures and related constants
 *
 ***************************************************************************/

#ifndef CAM_DEF_H
#define CAM_DEF_H

#if defined (_WIN64)
	#include<basetsd.h>
	#ifndef _64BIT_
		#define _64BIT_
	#endif
#endif


#ifdef _BIOS
	#define	FARPTR	__far
#else
	#define	FARPTR
#endif

#ifdef NT
  #pragma pack (1)
#endif /* NT */

#ifndef PACKED
	#if defined (__LINUX__) || defined (__FreeBSD__)			
	    #define PACKED __attribute__((packed))
	#else  
	    #define PACKED
	#endif
#endif

#ifdef U8
	#undef	U8
	#undef	U16
	#undef	U32
	#undef	U64
	#undef	PU8
	#undef	PU16
	#undef	PU32
	#undef	PU64
#endif
	#define U8 unsigned char
	#define U16 unsigned short

#ifdef	_64BIT_
	#if defined (__LINUX__) || defined (__FreeBSD__)			
		#define U32 unsigned int
		#define U64 unsigned long
	#else
		#define U32 unsigned long
		#define U64 ULONG64	// for win amd64
	#endif
#else
	#define U32 unsigned long
	#define U64 unsigned long
#endif
	


	#define PU8 U8 FARPTR *
	#define PU16 U16 FARPTR *
	#define PU32 U32 FARPTR *
	#define PU64 U64 FARPTR *


#ifdef	_64BIT_
	#define	PTR_SIZE	U64
#else
	#define	PTR_SIZE	U32
#endif


#ifdef	PVOID
	#undef	PVOID
#endif
	#define PVOID void FARPTR *

typedef struct _CAM_INFO {
		U8		VerMajor;
		U8		VerMinor;
		U8		VerOEM;
		U8		VerBuild;
		U16		CAMFlag;
		U16		DeviceID;
		U32		MinMemSize;
		U32		MaxMemSize;
		U32		PacketSize;
		U8		NumofATA;
		U8		NumofXOR;
		U8		NumofHDMA;
		U8		VirMemBase;
} PACKED CAM_INFO, PACKED FARPTR *PCAM_INFO;


typedef struct _ADAPTER_CONFIG_INFO {
	U8		Adapter_ID;
	U8		NoNCQAccess;
	U8		TapeOutVersion;
	U8		Reserved[1];
	PTR_SIZE	BaseAddress[6];
	PTR_SIZE	MemPhyAddress;	    
	U32		MemSize;
	PVOID		MemVirAddr;
}PACKED ADAPTER_CONFIG_INFO, PACKED FARPTR *PADAPTER_CONFIG_INFO;

/* definitions for TapeOutVersion */
#define	REVISION_1		0x01		/* second tapout and before */
#define	REVISION_2		0x02		/* third tapout */


typedef struct _CHNL_CONFIG {
		U8		bCh;
		U8		ChnlType;
		U8		ChnlSpeed;
		U8		CableType;
}PACKED CHNL_CONFIG, PACKED FARPTR *PCHNL_CONFIG;

/* definitions for ChnlType */
#define	CHNL_ATA	0x01		/* the channel is a parallel ATA channel */
#define	CHNL_SATA	0x02		/* the channel is a serial ATA channel */

/* definitions for ChnlSpeed */
#define	CHNL_U33	0x01		/* the channle support upto Ultra DMA mode 2 */
#define	CHNL_U100	0x02		/* the channel support upto Ultra DMA mode 5 */
#define	CHNL_U133 	0x04		/* the channel support Ultra DMA mode 6 	 */
#define	CHNL_U150 	0x08		/* the channel support Ultra150				 */

/* definitions for CableType */
#define	CABLE_40	0x01		/* 40-conductor cable 	*/
#define	CABLE_80	0x02		/* 80-condutor cable 	*/
#define	CABLE_SATA	0x03		/* SATA cable 			*/


typedef struct _ENCLOSURE_TYPE	PACKED ENCLOSURE_TYPE, PACKED FARPTR *PENCLOSURE_TYPE;

typedef struct  _DEV_CONFIG {
		U8		bCh; 					
		U8		bID;						
		U8		PIOMode;
		U8		MDMAMode;
		U8		UDMAMode;
		U8		QueueDepth;
		U16		DevFlag;
		U8		Model[22];
		U8		Reserved[2];
		U16		CylNum;
		U8		HeadNum;
		U8		SecPerTrk;
		U32		MAXLBAL;
		U32		MAXLBAH;
		U8		Features;
		U8		SerialNo[21];
		U8		FWVersion[10];
		PENCLOSURE_TYPE pEnclosure;
}PACKED DEV_CONFIG, PACKED FARPTR *PDEV_CONFIG;

/* definitions for DevFlag */
#define	DEV_NOTFOUND		0x0000	/* No drive exists */
#define	DEV_EXIST			0x0001	/* The drive exists */
#define	DEV_ATA_DEVICE		0x0002	/* This drive is a ATA device */
#define	DEV_ATAPI_DEVICE	0x0004	/* This drive is a ATAPI device */
#define	DEV_48BIT			0x0010	/* This drive with 48-bit LBA support */
#define	DEV_ATA5			0x0020	/* This device claims compliance with ATA-5 */
#define	DEV_NCQ				0x0040	/* The device supports native command queuing */
#define	DEV_QUEUED			0x0100	/* The device supports queued commands */
#define	DEV_FORCE_MEDIA		0x0200	/* The device supports forces the data be written to the device media before 
										command completion.
										If the device supports DEV_NCQ, it also supports 
										forces the data to be retrieved from the storage media. */
/* definitions for DevFlag internal used*/
#define DEV_INT_DRQ			0x8000	/* This device interrupts with the assertion of DRQ after receiving ATAPI Packet Command */

/* definitions for Features */
#define DEV_EN_SMART						0x01	/* SMART feature set has been enabled. */
#define DEV_EN_WRITECACHE					0x02	/* write cache has been enabled. */
#define DEV_EN_NCQ_DMASetupAuto				0x04	/* NCQ DMA Setup Auto-Activate optimization has been enabled. */
#define DEV_EN_NCQ_non_zero_offsets			0x08	/* NCQ non-zero buffer offsets in DMA Setup FIS enabled. */
#define DEV_SMART							0x10	/* supports SMART feature set. */
#define DEV_WRITECACHE						0x20	/* write cache supported. */
#define DEV_NCQ_DMASetupAuto				0x40	/* NCQ DMA Setup Auto-Activate optimization supported. */
#define DEV_NCQ_non_zero_offsets			0x80	/* NCQ supports non-zero buffer offsets in DMA Setup FIS */


typedef struct _CAMSG {
	U32		StartAddr;
	U16		wCount;
	U8		Reserved;				
	U8		CtrlFlag;					
} PACKED CAMSG, PACKED FARPTR *PCAMSG;

/* definitions for CtrlFlag */
#define EOT					0x80	/* End of the SG table */


typedef struct _DEV_TAG_DEVICE {
		U8		TagCmdFlag;
		U8		SEQID;
		U8		TCQTransferData;
		U8		Current_bID;
		U8		Support_device;
		U8		PM;
		U8		IsWDHardDrive;
		U8		CurrentXferTAG;
		U32		FreeTAG;
		U32		UsedTAG;
}PACKED DEV_TAG_DEVICE, PACKED FARPTR *PDEV_TAG_DEVICE;

#define MAX_TAG	32

/* definitions for TagCmdFlag */
#define TAG_SEQALLOC		bit(0)
#define TAG_DATAIDLE		bit(1)	
#define TAG_DATAIN			bit(2)	
#define TAG_DATAOUT		bit(3)	


typedef struct _ATA_TASK	PACKED ATA_TASK, PACKED FARPTR *PATA_TASK;	

struct _ATA_TASK {
	void (*callback)(PATA_TASK pTask);	/* call back function */
	U8		bCh; 					
	U8		bID;	
	U8		bCh2Chip;	/* internal use */
	U8		Reserved1;
	U16		ATAStatus;					
	U16		ATACmdFlag;
union {
	U8		Cdb[16];				
	struct {
			U8		bError;
			U8		bFeature;	
			U16		bCount;			
			U32		LBAl;		
			U32		LBAh;
			U8		bCmd;
			U8		bStatus;
			U8		bCmd2Chip;	/* internal use */
			U8		Reserved;	/* to align with Cdb */
		} PACKED ATACmd;
	} Cmd;
		U8		SGCount;
		U8		SenseInfoLength; 
		U16		Reserved3; 
		U32		DataTransferLength;
		PVOID	DataBuffer;
		PTR_SIZE	pSG;
		PVOID	SenseInfoBuffer; 
		PVOID	pTaskExtension;
};

/* definitions for ATAStatus */

#define ATA_PH_ERR			0x0001		/* indicate PH_ERR */
#define ATA_SH_ERR			0x0002		/* indicate SH_ERR */
#define ATA_DH_ERR			0x0004		/* indicate DH_ERR */

#define ATA_HTO_ERR			0x0010		/* indicate HTO_ERR */
#define ATA_TA_HBA_ERR		0x0020		/* indicate TA_HBA_ERR */
#define ATA_TA_DMA_CNT_ERR	0x0040		/* indicate TA_DMA_CNT_ERR */

#define	ATA_ERROR			0x0100		/* indicate this ATA_TASK is failed */
#define	ATA_ABORT			0x0200		/* indicate this ATA_TASK is aborted */
#define	ATA_TIMEOUT			0x0400		/* indicate this ATA_TASK timeout */

#define	ATAERR_OVERRUN		0x0800		/* indicate total count of S/G table is larger than HDD requires */
#define	ATAERR_UNDERRUN 	0x1000		/* indicate total count of S/G table is less than HDD requires */
#define	ATAERR_DRV			0x2000		/* check bError and bStatus of ATACmd or check sense data in SenseInfoBuffer to get the detail information */
#define	ATAERR_PCI			0x4000		/* Indicates that is due to PCI system error */

#define	ATAERR_HWERROR		(ATA_PH_ERR+ATA_SH_ERR+ATA_DH_ERR+ATA_HTO_ERR+ATA_TA_HBA_ERR+ATA_TA_DMA_CNT_ERR+ATAERR_OVERRUN+ATAERR_UNDERRUN+ATAERR_DRV+ATAERR_PCI)

/* definitions for ATACmdFlag */
#define PSEUDO_CMD			0x0001	/* indicate a pseudo command is in bCmd */
#define camDATA_XFER		0x0002	/* indicate this is a data transfer command */
#define camDATA_IN			0x0004	/* indicate this is a data in command */
#define ATA_DEVICE			0x0008	/* indicate specified device is a ATA device */
#define PIO_XFER			0x0010	/* indicate this is a data transfer command by PIO */
#define ATA_STATUS_RETURN 	0x0020 	/* indicate return ATA registers statuses when this command completed */
#define PSEUDO_TAG_CMD		0x0040	/* indicate a pseudo NCQ/QUEUED command is in bCmd */
/* if device support Native Command Queuing and specify the command ReadFPDMAQueued or WriteFPDMAQueued, 	*/
/* When set FORCE_MEDIA, forces the data to be retrieved from the storage media regardless of whether the 	*/
/* storage device holds the requested information in its buffers or cache or forces the data to be written	*/
/* to the storage media before completion status is indicated. 												*/
#define FORCE_MEDIA			0x0080  
#define DATA_ENCRYPT		0x0100  /* indicate data transfer will be encryption */

/* definitions for ATACmdFlag internal used*/
#define INTERNAL_EXIST		0x2000	/* indicate this task exist */
#define INTERNAL_SENSE		0x4000
#define SWAPBOX_CMD			0x8000	/* indicate this task is for swap box */

struct _ENCLOSURE_TYPE {
	U8		BOXID;
	U8		BOXType;
	U16		BOXLEDConfig;
	U8		BOXConfig;
	U8		NumofBay;
	U8		NumofFAN;
	U8		NumofSensor;
	U16		ChkPeriod;
	U16		Reserved;
};

/* definitions for BOXType */
#define	camNOBOX		0xFF		/* no box or bad box */
#define	SuperSwap		0x01
#define	SuperSwap1100	0x02
#define	SuperSwap4100	0x04
#define	PSS10398_1_CHNL	0x08		/* OEM - 1 channel no PWM */
#define	PSS10398_4_CHNL	0x10		/* OEM - 4 channel no PWM */
#define	camUNKNOWNBOX	0x00

#define SATAEnclosure 	(SuperSwap1100+SuperSwap4100+PSS10398_1_CHNL+PSS10398_4_CHNL)

/* definitions for BOXLEDConfig */
#define	WITH_STATUS_LED					0x0001
#define	WITH_FLASH_OF_STATUS_LED		0x0002
#define WITH_TURN_OFF_STATUS_LED		0x0004
#define	WITH_ACTIVE_LED					0x0008
#define	WITH_FLASH_OF_ACTIVE_LED		0x0010
#define WITH_TURN_OFF_ACTIVE_LED		0x0020
#define	WITH_Global_LED					0x0040
#define	WITH_FLASH_OF_Global_LED		0x0080
#define WITH_TURN_OFF_Global_LED		0x0100

/* definitions for BOXConfig */
#define	WITH_FAN		0x01		/* the enclosure has fan(s)*/
#define	WITH_KEY		0x02		/* the enclosure has key to turn on/off power */
#define	WITH_SENSOR		0x04		/* the enclosure has sensor(s) */
#define	MODE_198P		0x10		/* 198 or 198p mode */
#define	MODE_SF			0x20		/* SF mode */
#define	COMBO			0x80		/* has muitible bays */

typedef struct _ENCLOSURE_STATUS {
	U8		NeedCheck;
	U8		PowerState;
	U8		Reserved[2];
	U16		RemPeriod;
	U16		TEMPStatus;
	U16		TEMP2Status;
	U16		TEMP3Status;
	U32		FANStatus;
	U32		FAN2Status;
	U32		FAN3Status;
	U32		VOL3V;
	U32		VOL5V;
	U32		VOL12V;
} PACKED ENCLOSURE_STATUS, PACKED FARPTR *PENCLOSURE_STATUS;


/* definitions for PowerState */
#define	PowerOFF		0x01
#define	PowerON			0x00

/*-------------------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------------------*/

typedef struct _ADATER_CONFIG {
	
	U8		Adapter_ID;
	U8		NoNCQAccess;
	U8		TapeOutVersion;
	U8		Features;
	PTR_SIZE	ATABaseAddr;
	PTR_SIZE	XORBaseAddr;
	PTR_SIZE	HostBaseAddr;
	PTR_SIZE	MemIOBaseAddr;
	PTR_SIZE	MemPhyAddress;	    
	U32		MemSize;
	PVOID		MemVirAddr;
	U32		PLL_Parms;			/* for PDC20618/619/620 */	
	U32		EncryptionKey;
}PACKED ADAPTER_CONFIG, PACKED FARPTR *PADAPTER_CONFIG;

/* definitions for Features */
#define	WITH_EncryKEY		0x01		




/* Init or Reset channel, used for keep control flag */
typedef struct _RESET_CH_CONTROL {
	void		(*notify)(U8 bChannelNumber, U8 status);	/* notification function */
	U8		OwnerType;
	U8		run_count;
	U8		receiveStatus;
	U8		SEQID;
	U8		SEQCntCtrl;
	U8		Reserved[3];
}PACKED RESET_CH_CONTROL, PACKED FARPTR *PRESET_CH_CONTROL;

/* definition for RESET type */
#define	WAIT_INIT_CH		bit(0)
#define	WAIT_RESET_CH		bit(1)
#define	WAIT_TIMEOUT		bit(2)



typedef	struct _FLASH_CONFIG {
	U16		Flag;				
	U16		Flash_Type;			/* flash memory type */
	U32		Old_Timing;			/* keep old PLL or TBG clcok setting */
	U32		New_Timing;			/* timing during programming flash memory */
	PTR_SIZE	FlashMemAddr;			/* address to access flash memory */

}PACKED FLASH_CONFIG, PACKED FARPTR *PFLASH_CONFIG;
#define	FLASH_FOUND		bit(0)
#define FLASH_620		bit(1)			/* PDC20620/617 series ASIC */
#define FLASH_320		bit(2)			/* PDC20320/375 series ASIC */


typedef struct _PACKET_WAITQ{
	PVOID	NPA;					/* Next Packet Address*/
	PVOID	PTA;    				/* Parent Task Address, original Task Point */
	U16		Flag;
	U16		CountStamp;
}PACKED PACKET_WAITQ, PACKED FARPTR *PPACKET_WAITQ;

typedef struct _PACKET_HEAD{
	PVOID  NPA;					/* Next Packet Address*/
	PVOID  PTA;    				/* Parent Task Address, original Task Point */
	PVOID  PSG;    				/* SG virtual address used for this packet */
	U16    ModelType;			/* indicate ATA channel number */
	U16    Flag;
	U16		CountStamp;
/*#ifdef	_64BIT_*/
	U8	reserverd64[2];
/*#endif*/
	PTR_SIZE    Address;				/* packet physical address of itself */ 
}PACKED PACKET_HEAD, PACKED FARPTR *PPACKET_HEAD;
#define PACKET_HEAD_SIZE	sizeof(PACKET_HEAD)

/* definition for PACKET_HEAD */
#define	PACFLAG_ATAPI			bit(0)	/* indicate this packet is for ATAPI device */
#define	PACFLAG_ATAPI_PIO		bit(1)	/* indicate this packet is for ATAPI device PIO data transfer*/
#define	PACFLAG_NCQ			bit(2)	/* indicate this packet is for NCQ command */
#define	PACFLAG_QUEUED			bit(3)	/* indicate this packet is for QUEUED command */
#define PACFLAG_FLUSH			bit(4)	/* indicate this packet is for flush cache command */

#define	PACFLAG_SORT_PASS		bit(7)	/* indicate this packet is for not do sort */

/* definition for Sequence Counter Control Register*/
#define SEQ_FPDMA				bit (6) /* pass all device interrupt to PCI bus. */
#define SEQ_INT_MASK			bit (5) /* mask interrupt by sequence counter. */

#define PACKET_COUNTDOWN		256	//512


typedef struct _ATA_PACKET{

	U8	bControl;
	U8	bPMPort;
	U8	bSynSeqID;				/* synchronization sequence ID */
	U8	bDelaySeqID;			/* delay seqence ID           */
                                              
	U32	ulPSG;					/* point to SG table		*/
	U32	ulNCA;					/* Pointer to Next ATA Packet  */

	U8	bATRL[MAXREGCNT];

}PACKED ATA_PACKET, PACKED FARPTR *PATA_PACKET;


/* definition for ATA/ATAPI packet */
#define ATACTRL_ENCRYPT_ENABLE	bit(1)		/* encryption enable */
#define ATACTRL_DMAREAD			bit(2)		/* DMA Read command*/
#define ATACTRL_NONDATA			bit(3)		/* Non-Data command */
#define ATACTRL_DELAY			bit(4)
#define ATACTRL_PAUSE			bit(5)
#define ATACTRL_BOX				bit(6)		/* Command is for enclusure control */
#define NCQFPDMA_CMD			bit(7)

#define CAM_PRD_TBL_SIZE 	0x108			/* Max. 33 PRD entries */
/* Important: the addresses must be DWord boundary. */
typedef struct _CAM_ATA_SG {
	U8	ATA_SG[CAM_PRD_TBL_SIZE];
	U8	PadByte[4];				/* for fixing the ASIC bug about ATAPI device commands */
	PTR_SIZE	Address;				/* packet physical address of itself */ 
}PACKED CAM_ATA_SG, PACKED FARPTR *PCAM_ATA_SG;


typedef struct _CAM_PACKET{
	PACKET_HEAD	PacketHead;
	ATA_PACKET		ATAPacket;
}PACKED CAM_PACKET, PACKED FARPTR *PCAM_PACKET;


typedef struct _CAM_ATA_PACKET{
	PACKET_WAITQ		PacketHead;
}PACKED CAM_ATA_PACKET, PACKED FARPTR *PCAM_ATA_PACKET;



#define ATA_PAC_SIZE sizeof(CAM_PACKET)

#define ATA_SG_SIZE sizeof(CAM_ATA_SG)

#define RECE_PAC_SIZE sizeof(CAM_ATA_PACKET)

typedef enum {
    ATA0 = 0x00,
    ATA1 = 0x01,
    ATA2 = 0x02,
    ATA3 = 0x03,    
    XOR = 0x04
} ModType;

typedef struct _SEQ_INFO
{
	PVOID	pCurPAC;			/* point to current packet */
	PVOID	SEQID2PacketHead;
	PVOID	SEQID2PacketTail;
	ModType	Owner;				/* which module the SEQ belongs to */
	U8		PACCNT;				/* current packet count in this SEQ ID */
	U8		GO;
}PACKED SEQ_INFO, PACKED FARPTR *PSEQ_INFO;


typedef enum {
    ATA_PAC = 0x00,
    XOR_PAC = 0x01,
    HDMA_PAC = 0x02
} PAC_TYPE, PAC_Q_TYPE;

typedef struct _ATA_QUEUE	PACKED ATA_QUEUE, PACKED FARPTR *PATA_QUEUE;
struct _ATA_QUEUE{
	PCAM_ATA_PACKET pATAPacket;
	PATA_QUEUE		p_ata_queue_next;
};

typedef struct _ATA_MODULE_PACKET_COUNT{
	U8 ATAModule[MAX_ATA_CHNL];
}PACKED ATA_MODULE_PACKET_COUNT, PACKED FARPTR *PATA_MODULE_PACKET_COUNT;

typedef struct _MODULE_TIMEOUT{
	U32 ATACommand;
	U32 ATAPICommand;
	U32 XORCommand;
	U32 BOX_Check;
	U32 Port_Multiplier_Check;
}PACKED MODULE_TIMEOUT, PACKED FARPTR *PMODULE_TIMEOUT;

typedef struct _CAM_IDENTIFY_DATA {

	U16 GeneralConfiguration;			/* 00 00	*/
	U16 NumberOfCylinders;              /* 02  1	*/
	U16 Reserved1;                      /* 04  2	*/
	U16 NumberOfHeads;                  /* 06  3	*/
	U16 UnformattedBytesPerTrack;       /* 08  4	*/
	U16 UnformattedBytesPerSector;      /* 0A  5	*/
	U16 SectorsPerTrack;				/* 0C  6	*/
	U16 VendorUnique1[3];               /* 0E  7-9	*/
	U16 SerialNumber[10];               /* 14  10-19	*/
	U16 BufferType;						/* 28  20	*/
	U16 BufferSectorSize;				/* 2A  21	*/
	U16 NumberOfEccBytes;				/* 2C  22	*/
	U16 FirmwareRevision[4];			/* 2E  23-26	*/
	U16 ModelNumber[20];				/* 36  27-46	*/
	U8  MaximumBlockTransfer;			/* 5E  47	*/
	U8  VendorUnique2;					/* 5F	*/
	U16 DoubleWordIo;					/* 60  48	*/
	U16 Capabilities;					/* 62  49	*/
	U16 Reserved2;						/* 64  50	*/
	U8  VendorUnique3;					/* 66  51	*/
	U8  PioCycleTimingMode;				/* 67	*/
	U8  VendorUnique4;					/* 68  52	*/
	U8  DmaCycleTimingMode;				/* 69	*/
	U16 FieldsValid;					/* 6A  53	*/
	U16 NumberOfCurrentCylinders;		/* 6C  54	*/
	U16 NumberOfCurrentHeads;			/* 6E  55	*/
	U16 CurrentSectorsPerTrack;			/* 70  56	*/
	U16 CurrentSectorCapacity1;			/* 72  57-58	*/
	U16 CurrentSectorCapacity2;			/* 72  57-58	*/
	U16 CurrentMultiSectorSetting;		/*     59	*/
	U16 UserAddressableSectors1;		/*     60-61	*/
	U16 UserAddressableSectors2;		/*     60-61	*/
	U16 SingleWordDMAModes;				/*     62	*/
	U16 MultiWordDMAModes;				/*     63	*/
	U16 AdvancedPIOModes;				/*     64	*/
	U16 MinimumMWXferCycleTime;			/*     65	*/
	U16 RecommendedMWXferCycleTime;		/*     66	*/
	U16 MinimumPIOCycleTime;			/*     67	*/
	U16 MinimumPIOCycleTimeIORDY;		/*     68	*/
	U16 Reserved5[4];					/*     69-72	*/
	U16 Word73;							/*     73	*/
 	U16 Word74;							/*     74	*/
	U16 QueueDepth;						/*     75	*/
	U16 SerialATAcapabilities;			/*     76	*/
	U16 Word77;							/*     77	*/
	U16 SerialATAFeaturesSupported;		/*     78	*/
	U16 SerialATAFeaturesEnabled;		/*     79	*/
 	U16 MajorRevision;					/*     80	*/
	U16 MinorRevision;					/*     81	*/
	U16 CommandSetSupport1;				/*     82	*/
	U16 CommandSetSupport2;				/*     83	*/
	U16 CommandSetSupport3;				/*     84	*/
	U16 CommandSetSupport4;				/*     85	*/
	U16 CommandSetSupport5;				/*     86	*/
	U16 CommandSetSupport6;				/*     87	*/
	U16 CommandSetSupport7;				/*     88	*/
	U16 Reserved7[11];					/*     89 - 99	*/
	U16 MaxLBA1;						/*     100	*/
	U16 MaxLBA2;						/*     101	*/
	U16 MaxLBA3;						/*     102	*/
	U16 MaxLBA4;						/*     103	*/
	U16 Reserved8[22];					/*     104 - 125	*/
	U16 LastLUNID;						/*	   126 (for ATAPI device )	*/
	U16 SpecialFunctionsEnabled;		/*     127	*/
	U16 Reserved9[128];					/*     128-255	*/
	
}PACKED CAM_IDENTIFY_DATA, PACKED FARPTR *PCAM_IDENTIFY_DATA;

#define ATA_IDENTIFY_SIZE sizeof(CAM_IDENTIFY_DATA)

#define	LEDNOCHANGE	0xFF

/* definitions for LED type */
#define	ActiveLED		bit(0)
#define	StatusLED		bit(1)
#define	GlobalLED		bit(2)

/* definitions for on/off of ActiveLED/GlobalLED */
#define	LEDOFF			0x00
#define	LEDON			0x01
/* for LED Color */
#define	LED_Red			0x02
#define	LED_Green		0x03
#define	LED_Amber		0x04
#define	LED_MASK		0x03

/* definitions for LED flash */
#define	FASTFLASH		0x00
#define	SLOWFLASH		0x01
#define	NOFLASH			0x02


/* definitions for flash */
#define	HDActiveOFF			0x00
#define	HDActiveON			0x01

#define	MAXDELAYFORI2C		0x800

#define	BayShiftBits		4
/* definition of DEVICE ID*/
#define SBOX_4				bit(0)
#define SBOX_NO_PWM			bit(1)

#define	NOSBOX				0x00

/* definitions for BOXConfig of SBOX_Type*/
#define	SBOXEXIST					bit(0)
#define	SBOXWITHPWM					bit(1)
#define	SBOXWITHPWMUNASSIGNED		bit(2)
#define	SBOXCOMBO					bit(7)		/* has muitible bays */

/* definition of 10398 Chip control register */
#define	PWMCOMPARE				bit(2)
#define	PWMCOMPARERESULT		bit(3)

typedef struct _CAM_I2C_QUEUE	PACKED CAM_I2C_QUEUE, PACKED FARPTR *PCAM_I2C_QUEUE;

struct _CAM_I2C_QUEUE
{
	void (*callback)(PCAM_I2C_QUEUE pI2C_QUEUE);	/* call back function */
	U32		Data;
	U8		Data_CNT;
	U8 		b_ID;
	U8		LED_Type;	/* ActiveLED/StatusLED/GlobalLED */
	U8		reserved;
};

typedef struct _CAM_SBOX_LED
{
	union {
		U8		ActiveLed;	/* LEDON/LEDOFF */
		U8		StatusLed;	/* LEDOFF/LED_Red/LED_Green/LED_Amber */
		U8		GlobalLed;	/* LEDOFF/LED_Red/LED_Green/LED_Amber */
	} LED;
	U8		Flash;	/* NOFLASH/SLOWFLASH/FASTFLASH */
}PACKED CAM_SBOX_LED, PACKED FARPTR *PCAM_SBOX_LED;

typedef struct _CAM_HDAccess_LED_Buffer
{
	CAM_SBOX_LED Buffer_LED;
	U8 b_ID;
	U8 UsedFlag;
	U8 RetryCount;
	U8 NeedRetryFlag;
	U8 reserved[2];
}PACKED CAM_HDAccess_LED_Buffer, PACKED FARPTR *PCAM_HDAccess_LED_Buffer;

typedef struct _CAM_Ctrl_LED
{
	union {
		U8		StatusLed;	/* LEDOFF/LED_Red/LED_Green/LED_Amber */
		U8		ActiveLed;	/* LEDON/LEDOFF */
		U8		GlobalLed;	/* LEDOFF/LED_Red/LED_Green/LED_Amber */
	} LED;
	U8		Flash;		/* NOFLASH/SLOWFLASH/FASTFLASH */
	U8		LED_Type;	/* ActiveLED/StatusLED/GlobalLED */
}PACKED CAM_Ctrl_LED, PACKED FARPTR *PCAM_Ctrl_LED;

typedef struct _CAM_HDAccess_LED
{
	U8		Flash;		/* NOFLASH/SLOWFLASH/FASTFLASH */
	U8		LED_OnOff;	/* LEDON/LEDOFF */
}PACKED CAM_HDAccess_LED, PACKED FARPTR *PCAM_HDAccess_LED;

typedef struct _CAM_HDAccess_LED_All
{
	CAM_HDAccess_LED Active_LED;
	CAM_HDAccess_LED Status_LED;
	CAM_HDAccess_LED Global_LED;
}PACKED CAM_HDAccess_LED_All, PACKED FARPTR *PCAM_HDAccess_LED_All;

typedef struct _SBOX_Type
{
	U8		BOXID;
	U8		BOXType;
	U8		BOXConfig;
	U8		BOXSensorNum;
	union {
		U8		StatusLEDColor;	/* LED_Red/LED_Green/LED_Amber */
		U8		GlobalLEDColor;	/* LED_Red/LED_Green/LED_Amber */
	} LEDColor;	
	union {
		U8		GlobalFlash;	/* NOFLASH/SLOWFLASH/FASTFLASH */
	} Flash;		
	U8		reserved[2];
}PACKED SBOX_Type, PACKED FARPTR *PSBOX_Type;

/* Serial Bus */
typedef struct _CAM_SBUS_TASK	PACKED CAM_SBUS_TASK, PACKED *PCAM_SBUS_TASK;
struct _CAM_SBUS_TASK {
	void (*callback)(PCAM_SBUS_TASK pTask);	/* call back function */
	PVOID		pTaskExtension;
	U8			TaskSize;
	U8			SubFunCode;
	U8			SlaveAddress;
	U8			Reserved;
	U16 		WriteByteCount;
	U16 		ReadByteCount;
	PVOID		pWriteBuffer;			
	PVOID		pReadBuffer;			
};

typedef struct _SERIALBUS_ADDR {
	PTR_SIZE		MemPhyAddress[1];	    
	PVOID			MemVirAddr[1];
	PTR_SIZE		MemPollPhyAddress[2];	    
	PVOID			MemPollVirAddr[2];		
	PVOID			MemVirCurrent;	
}PACKED SERIALBUS_ADDR, PACKED FARPTR *PSERIALBUS_ADDR;


/*==========================================================================================*/
/* Error Injection */
#ifdef _ERR_INJECT_

#define MAX_ERRINJ_TASK	64

typedef struct  _ERR_CONFIG {
		U8		bCh; 					
		U8		bID;
		U8		TaskCount;	
		U8		Flag;	
		U16		ErrType;
		U8		reserver[2];
		U32		HitLBAl;	
		U32		HitLBAh;	
}PACKED ERR_CONFIG, PACKED FARPTR *PERR_CONFIG;


/* definitions for ErrType */
#define ERR_CLEAR_HIT			bit(0)				
#define ERR_RESERVE_HIT			bit(1)
#define ERR_CLEAR_OTHER			bit(2)
#define ERR_AUTO_HIT_LBA		bit(3)

/* definitions for ErrType */

#define ERRINJ_PH_ERR			ATA_PH_ERR		
#define ERRINJ_SH_ERR			ATA_SH_ERR		
#define ERRINJ_DH_ERR			ATA_DH_ERR		

#define ERRINJ_HTO_ERR			ATA_HTO_ERR		
#define ERRINJ_TA_HBA_ERR		ATA_TA_HBA_ERR		
#define ERRINJ_TA_DMA_CNT_ERR	ATA_TA_DMA_CNT_ERR	
#define	ERRINJ_TIMEOUT			ATA_TIMEOUT		

#define	ERRINJ_OVERRUN			ATAERR_OVERRUN		
#define	ERRINJ_UNDERRUN 		ATAERR_UNDERRUN		
#define	ERRINJ_PCI				ATAERR_PCI		

#define	ERRINJ_ATA_IDNF		0x3
#define	ERRINJ_ATA_UNC			0x5
#define	ERRINJ_ATA_CRC			0x9
#define	ERRINJ_ATA_DF			0x30
#define	ERRINJ_ATA_UNC_W		0x50

#define	ERRINJ_SATA_PLUG		0x6
#define	ERRINJ_SATA_UNPLUG		0x7


#define	PACFLAG_ERR_INJECT			bit(7)	/* indicate this packet is for Error Injection */
#define	PACFLAG_TIMEOUT_INJECT		bit(8)	/* indicate this packet is for Error Injection */

/* each channel Err Injected task manager */
typedef struct  _ERRINJ_TASK_TABLE {
		U16		head; 					
		U16		tail;
		U16		status;	
		U16		count;	
}PACKED ERRINJ_TASK_TABLE, PACKED FARPTR *PERRINJ_TASK_TABLE;

#define	EI_NEED_COUNT_DOWN	bit(0)

/* content of Err Injected Task */
typedef struct  _ERRINJ_TASK {
		U8		aa; 					
		U8		UsedFlag;	
		U16		NextTaskId;	
		ERR_CONFIG	ErrInjTask_Config;	
}PACKED ERRINJ_TASK, PACKED FARPTR *PERRINJ_TASK;

/*==========================================================================================*/
#endif


#ifdef NT
  #pragma pack ()
#endif /* NT */

#endif
