/***************************************************************************
 *
 * Copyright (c) 2002 Promise Technology, Inc.  All rights reserved.
 * No part of this document may be reproduced or transmitted in any form or
 * by any means, electronic or mechanical, for any purpose, without the
 * express written permission of Promise Technology, Inc.
 *
 *
 * cam_swap.c - CAM Enclosure functions
 *
 ***************************************************************************/
#include "cam_con.h"
#include "cam_def.h"
#include "cam_imp.h"
#include "cam_macr.h"
#include "cam_gb.h"
#include "cam_exp.h"

U8 EnclosureEnterState(PENCLOSURE_TYPE pBox);
void EnclosureExitState(PENCLOSURE_TYPE pBox);
void Get_Box_Status(PATA_TASK pATATask);
U16 ReadWinBondID_W83783S(PTR_SIZE BaseAddress );
void SetupFanDivisor(PTR_SIZE BaseAddress, U8 Divisor);
U32 ReadFanStatus(PTR_SIZE BaseAddress );
U16 ReadTemperature(PTR_SIZE BaseAddress);
U32 ReadVoltage5V(PTR_SIZE BaseAddress);
U32 ReadVoltage12V(PTR_SIZE BaseAddress);
void SBoxSetupAllFanDivisor(U8 Adapter_ID, U8 bID, U8 Divisor);
U32 SBoxReadFanStatus(U8 Adapter_ID, U8 bID);
U16 SBoxReadTemperature(U8 Adapter_ID, U8 bID);
U32 SBoxReadVoltage5V(U8 Adapter_ID, U8 bID);
U32 SBoxReadVoltage12V(U8 Adapter_ID, U8 bID);
U32 SBoxReadVoltage3V(U8 Adapter_ID, U8 bID);
U32 SBoxReadFan2Status(U8 Adapter_ID, U8 bID);
U32 SBoxReadFan3Status(U8 Adapter_ID, U8 bID);
U16 SBoxReadTemperature2(U8 Adapter_ID, U8 bID);
U16 SBoxReadTemperature3(U8 Adapter_ID, U8 bID);
U8 Get_SataEnclosure_Datas(U8	bChannelNumber);
void Get_SataBox_Status(PCAM_I2C_QUEUE pI2C_QUEUE);
void (*Chk_BOX_Function[])(void);
/*U8 Data2SerialBus(U8 Adapter_ID, U32 WriteData);*/

U8 Get_Enclosure_Datas(U8	bChannelNumber)
{
	PTR_SIZE	BaseAddress = ATABaseAddr[bChannelNumber];
	PENCLOSURE_TYPE pBox = &gBOXConfig[bChannelNumber];
	PENCLOSURE_STATUS pStatus = &gBOXStatus[bChannelNumber];	
	U8	bData;
	U8	Insertstate=0;
		
	if (EnclosureEnterState(pBox) == TRUE ) {
		
		if ( pBox->BOXConfig & MODE_SF )
		{
			bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
			pStatus->PowerState = ( bData & bit(3) )? PowerON : PowerOFF;
			Insertstate = (U8)(bData & bit(1));
		}
		/* only when PowerOn of box, FAN RPM is available*/
		if (pStatus->PowerState == PowerON)
			pStatus->FANStatus = ReadFanStatus(BaseAddress );
		else
			pStatus->FANStatus = 0;
		
		/* only when box inserted, temp is available*/
		if (!Insertstate)
			pStatus->TEMPStatus = ReadTemperature(BaseAddress);	
		else
			pStatus->TEMPStatus = 0;
		
		pStatus->RemPeriod = pBox->ChkPeriod;
		pStatus->VOL5V = ReadVoltage5V(BaseAddress);	
		pStatus->VOL12V = ReadVoltage12V(BaseAddress);
		EnclosureExitState(pBox);
		return TRUE;
	}
	return FALSE;
}

U8 CAM_Check_Device_SBOX_PWM(U8 bChannelNumber)
{
	U8 i;
	U16	I2cDelay;
	U8 bData;
	U8 PWMflag;
	U32 data=0;
	U8 Adapter_ID = bChannelNumber/MAX_ATA_CHNL;
	
	if (I2CBusy[Adapter_ID] == TRUE) {
		/* mask ERR & Finish and  */
		WritePortWord(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, 0x0F00);	
			
		for (I2cDelay=0;I2cDelay<MAXDELAYFORI2C;I2cDelay++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (bData & 0x0F)	/* FINISH or ERR */
			{
				CAM_ResetSerialBusMachine(Adapter_ID);
				break;
			}			
		}
				
		/* call callback instead ISR */
		I2CQueueCurrent[Adapter_ID].callback(&I2CQueueCurrent[Adapter_ID]);
	}
	
	I2CBusy[Adapter_ID] = TRUE;
	for (i=0;i<MAX_ATA_CHNL;i++) {
		if (!(SBOXType[Adapter_ID][i].BOXConfig & SBOXWITHPWMUNASSIGNED))
			continue;
	
		/* 1.	set PWM offset register for desired enclosure ID (m). */
		CAM_SerialBusWriteDATAPolling(Adapter_ID,((5 << 8) | ((0x0A+0x10*i)<<1)),2);

		/* for SW4100, set the PWM compare bit = 1 for desired enclosure ID (m). */
		bData=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x09+0x10*i)<<1)+1),1);	
		bData |= PWMCOMPARE;
		data = (U32)bData;
		CAM_SerialBusWriteDATAPolling(Adapter_ID,((data << 8) | ((0x09+0x10*i)<<1)),2);	
	
		/* 2.	reset the PWM compare bit = 0 and result bit = 1 for desired enclosure ID (m). */
		bData=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x09+0x10*i)<<1)+1),1);	
		bData &= ~PWMCOMPARE;
		bData |= PWMCOMPARERESULT;		
		data = (U32)bData;
		CAM_SerialBusWriteDATAPolling(Adapter_ID,((data << 8) | ((0x09+0x10*i)<<1)),2);	
		/* 3.	wait for 20ms to admit the value of HD idle current is wrote to 'Width0' */
		camStallExecution(2);

		/* 4.	set the PWM compare bit = 1 for desired enclosure ID (m). */
		bData=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x09+0x10*i)<<1)+1),1);	
		bData |= PWMCOMPARE;
		data = (U32)bData;
		CAM_SerialBusWriteDATAPolling(Adapter_ID,((data << 8) | ((0x09+0x10*i)<<1)),2);	
		/* 5.	software send ATA command to HD at SATA channel (n). */
		/* disable INTA here, it will be re-enable when CAM use SEQ 0 for packets */
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECICR, 0x0);  /* that the drive INT pass to SEQ 0*/
		WritePortUchar( HostBaseAddr[Adapter_ID], 0, 0x20); /* but mask SEQ 0 INT */
			
		/* read verify */		
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorCount , 0x00); /* 256 sectors */
		/* check lba address large than 2^28 */		
		if (gDrvConfig[bChannelNumber][0].MAXLBAH | (gDrvConfig[bChannelNumber][0].MAXLBAL & 0xF0000000) ) {
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorNumber, 0xFF);
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECylinderLow, 0xFE);
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECylinderHigh,(U8)(0xFF-i));
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead, (U8)(0xE0 |0x0F) );
		}
		else {
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorNumber,(U8)(gDrvConfig[bChannelNumber][0].MAXLBAL-0x100-i*0x10000));
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECylinderLow,(U8)((gDrvConfig[bChannelNumber][0].MAXLBAL-0x100-i*0x10000) >> 8));
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECylinderHigh,(U8)((gDrvConfig[bChannelNumber][0].MAXLBAL-0x100-i*0x10000) >> 16));
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead, (U8)(0xE0 |((((gDrvConfig[bChannelNumber][0].MAXLBAL-0x100-i*0x10000) >> 24))&0x0F)) );
		}
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECommand, IDE_COMMAND_READ_VERIFY_SECTORS);
		WaitForINT(ATABaseAddr[bChannelNumber]);

		/* back */
		/* read verify */		
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorCount , 0x00);
	
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorNumber,0);
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECylinderLow,0);
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECylinderHigh,(U8)(0+i));
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead, 0xE0);
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDECommand, IDE_COMMAND_READ_VERIFY_SECTORS);
		WaitForINT(ATABaseAddr[bChannelNumber]);
				
		/* 6.	wait for 20ms to admit the value of HD active current is wrote to 'Counter', and PWM result bit is wrote. */
		camStallExecution(2);
		/* 7.	read back the PWM result bit from desired enclosure ID (m). */
		PWMflag=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x09+0x10*i)<<1)+1),1);	

		/* clear PWM result bit and compare bit */
		bData=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x09+0x10*i)<<1)+1),1);	
		bData &= ~PWMCOMPARE;
		bData |= PWMCOMPARERESULT;
		data = (U32)bData;
		CAM_SerialBusWriteDATAPolling(Adapter_ID,((data << 8) | ((0x09+0x10*i)<<1)),2);	


		if (PWMflag & PWMCOMPARERESULT){ /* PWM result TRUE */
			/* build map SBOXToCh table */
			gBOXConfig[bChannelNumber].BOXID=SBOXType[Adapter_ID][i].BOXID;
			gBOXConfig[bChannelNumber].BOXType = SBOXType[Adapter_ID][i].BOXType;
			gBOXConfig[bChannelNumber].NumofFAN = SBOXType[Adapter_ID][i].BOXSensorNum;
			gBOXConfig[bChannelNumber].NumofSensor = SBOXType[Adapter_ID][i].BOXSensorNum;
			if (SBOXType[Adapter_ID][i].BOXConfig & SBOXCOMBO) {
				gBOXConfig[bChannelNumber].BOXConfig |= COMBO;
				gBOXConfig[bChannelNumber].BOXLEDConfig = (WITH_STATUS_LED | WITH_FLASH_OF_STATUS_LED | WITH_ACTIVE_LED | WITH_FLASH_OF_ACTIVE_LED | WITH_TURN_OFF_ACTIVE_LED | WITH_Global_LED | WITH_FLASH_OF_Global_LED | WITH_TURN_OFF_Global_LED);
				gBOXConfig[bChannelNumber].NumofBay = MAX_BAY_PER_BOX;
			}
			else {
				gBOXConfig[bChannelNumber].BOXLEDConfig = (WITH_STATUS_LED | WITH_FLASH_OF_STATUS_LED | WITH_ACTIVE_LED | WITH_FLASH_OF_ACTIVE_LED | WITH_TURN_OFF_ACTIVE_LED);
				gBOXConfig[bChannelNumber].NumofBay = 1;				
			}
			
			/* check sensor */
			if (ReadWinBondID_W83782D(Adapter_ID,i) == WINBONDW83782DID)
				gBOXConfig[bChannelNumber].BOXConfig |= WITH_SENSOR;
			else
				gBOXConfig[bChannelNumber].BOXConfig &= ~WITH_SENSOR;
						
			ChnlMapToSBOXChnl[bChannelNumber]=i;
									
			SBOXType[Adapter_ID][i].BOXConfig &= ~SBOXWITHPWMUNASSIGNED;
			
			I2CBusy[Adapter_ID] = FALSE;
			CAM_StartI2C(Adapter_ID);
			return (gBOXConfig[bChannelNumber].BOXID);
		}
	}
	I2CBusy[Adapter_ID] = FALSE;
	CAM_StartI2C(Adapter_ID);
	gBOXConfig[bChannelNumber].BOXType = camNOBOX;
	gBOXConfig[bChannelNumber].BOXID = 0xFF;
	return 0xFF;
}


U8 CAM_SBOX_EnumEnclosure(U8 bChannelNumber, PENCLOSURE_TYPE pBox)
{
	U8 i;
	U8 Adapter_ID = bChannelNumber/MAX_ATA_CHNL;

	if (gChnlDrvControlConfig[bChannelNumber].Support_device >1)
		return 0xFF;
			
	if ( gBOXConfig[bChannelNumber].BOXType != camUNKNOWNBOX ) {
		*pBox =  *(&gBOXConfig[bChannelNumber]);
		return(pBox->BOXID);
	}
	

	/* box no PWM and map table had built */
	if ( (gBOXConfig[bChannelNumber].NumofBay == 1)
	|| (gBOXConfig[bChannelNumber].NumofBay == 4)) {
		U8 BoxId=CAM_ATA_Chnl_OutMAP(bChannelNumber)%MAX_ATA_CHNL;
		gBOXConfig[bChannelNumber].BOXType = SBOXType[Adapter_ID][BoxId].BOXType;
		gBOXConfig[bChannelNumber].NumofSensor = SBOXType[Adapter_ID][BoxId].BOXSensorNum;
		gBOXConfig[bChannelNumber].NumofFAN = SBOXType[Adapter_ID][BoxId].BOXSensorNum;
	}
	else{ /* box have PWM and map table had not built */

		/* check the attached drive of this Enclosure is exist ? */
		if (!(gDrvConfig[bChannelNumber][0].DevFlag & DEV_EXIST)) {
			gBOXConfig[bChannelNumber].BOXType = camUNKNOWNBOX;
			return 0xFF;		
		}
	
		for (i=0;i<MAX_ATA_CHNL;i++) {
			if (SBOXType[Adapter_ID][i].BOXConfig & SBOXWITHPWMUNASSIGNED)
				break;
		}
		
		if ( i>=MAX_ATA_CHNL ) {
			gBOXConfig[bChannelNumber].BOXType = camUNKNOWNBOX;
			return 0xFF;
		}
		if(CAM_Check_Device_SBOX_PWM(bChannelNumber) == 0xFF)
			return 0xFF;
	}
	
	gBOXConfig[bChannelNumber].BOXConfig |= WITH_KEY;
	/* set up monitor timer for enclusure */
	gBOXConfig[bChannelNumber].ChkPeriod = DEF_CHKSTATETIME * 10; /* 30s */
	/* get the first status of box */
	gBOXStatus[bChannelNumber].NeedCheck = TRUE;
	if(gBOXConfig[bChannelNumber].BOXConfig & WITH_SENSOR){	
		gBOXConfig[bChannelNumber].BOXConfig |= WITH_FAN; 
		SBoxSetupAllFanDivisor(Adapter_ID, ChnlMapToSBOXChnl[bChannelNumber], DEF_FANDIV);
		Get_SataEnclosure_Datas(bChannelNumber);
		if (!gBOXStatus[bChannelNumber].TEMPStatus) {
			gBOXConfig[bChannelNumber].BOXConfig &= ~WITH_SENSOR;
			gBOXConfig[bChannelNumber].NumofSensor = 0;
		}
	}
	else{	
		gBOXConfig[bChannelNumber].NumofSensor= 0;
		gBOXConfig[bChannelNumber].NumofFAN = 0;		
	}
	BOX_Timeout_TimerID[bChannelNumber] = camTimer(Adapter_ID,CAMTimeoutTimes[Adapter_ID].BOX_Check, Chk_BOX_Function[bChannelNumber]);	
	*pBox =  *(&gBOXConfig[bChannelNumber]);				
	return(pBox->BOXID);
}
	
U8 CAM_EnumEnclosure(U8 bChannelNumber, PENCLOSURE_TYPE pBox)
{
	CAM_Ctrl_LED Ctrl_LED;
	
	bChannelNumber=CAM_ATA_Chnl_InMAP(bChannelNumber);
	
	if (gChnlConfig[bChannelNumber].bCh == camCHNL_NOT_FOUND)
		return 0xFF;
		
	if (gChnlConfig[bChannelNumber].ChnlType == CHNL_SATA )
		return (CAM_SBOX_EnumEnclosure(bChannelNumber, pBox));
			
	if ( gBOXConfig[bChannelNumber].BOXType != camUNKNOWNBOX ) {
		*pBox =  *(&gBOXConfig[bChannelNumber]);
		return(pBox->BOXID);
	}
	
	/* first time to enumerate the enclosure */
	gBOXConfig[bChannelNumber].BOXID = bChannelNumber;
	gBOXConfig[bChannelNumber].BOXConfig = 0;
	if (EnclosureEnterState(&gBOXConfig[bChannelNumber]) == TRUE ) {
		EnclosureExitState(&gBOXConfig[bChannelNumber]);		

		/* default color is amber */
		Ctrl_LED.LED.StatusLed=LED_Amber;
		Ctrl_LED.Flash=LEDNOCHANGE;
		Ctrl_LED.LED_Type=StatusLED;
		CAM_CtrlLED(CAM_ATA_Chnl_OutMAP(bChannelNumber), &Ctrl_LED);
		
		/* setup relative parameters */
		EnclosureEnterState(&gBOXConfig[bChannelNumber]); /* get in by NCS way */
		/* read ID of sensor IC (Winbond H/W Monitoring IC) */
		/* if no find such IC, return number of sensor = 0 */
		if ( ReadWinBondID_W83783S(ATABaseAddr[bChannelNumber]) == WINBONDW83783SID) {
			SetupFanDivisor(ATABaseAddr[bChannelNumber], DEF_FANDIV );	
		}
		else
		{
			gBOXConfig[bChannelNumber].BOXConfig &= ~WITH_SENSOR;
			gBOXConfig[bChannelNumber].NumofSensor = 0;
		}
		EnclosureExitState(&gBOXConfig[bChannelNumber]);		
		
		/* set up monitor timer for enclusure */
		gBOXConfig[bChannelNumber].ChkPeriod = DEF_CHKSTATETIME * 10; /* 30s */
	
		/* get first status of swap box */
		if ( gBOXConfig[bChannelNumber].NumofSensor ) {
			gBOXStatus[bChannelNumber].NeedCheck = TRUE;
			boxATATask[bChannelNumber].bCh = bChannelNumber;
			Get_Enclosure_Datas(bChannelNumber);
			if (!gBOXStatus[bChannelNumber].TEMPStatus) {
				gBOXConfig[bChannelNumber].BOXConfig &= ~WITH_SENSOR;
				gBOXConfig[bChannelNumber].NumofSensor = 0;
			}
		}

		BOX_Timeout_TimerID[bChannelNumber] = camTimer( ((U8)( bChannelNumber/MAX_ATA_CHNL)),CAMTimeoutTimes[bChannelNumber/MAX_ATA_CHNL].BOX_Check, Chk_BOX_Function[bChannelNumber]);
	}
	else {
		gBOXConfig[bChannelNumber].BOXID = 0xFF;	/* no box */
		gBOXConfig[bChannelNumber].BOXType = camNOBOX;
	}
	
	*pBox =  *(&gBOXConfig[bChannelNumber]);
	return(pBox->BOXID);

}

void Get_Box_Status(PATA_TASK pATATask)
{
	U8	bChannelNumber = pATATask->bCh2Chip;
	U8	bData;
	PTR_SIZE	BaseAddress = ATABaseAddr[bChannelNumber];
	PENCLOSURE_TYPE pBox = &gBOXConfig[bChannelNumber];
	PENCLOSURE_STATUS pStatus = &gBOXStatus[bChannelNumber];
	U8 AdapterNumber=bChannelNumber/MAX_ATA_CHNL;
	
	if( pATATask->ATAStatus & ATA_ABORT ) {
		BOX_Timeout_TimerID[bChannelNumber] = camTimer(AdapterNumber,CAMTimeoutTimes[bChannelNumber/MAX_ATA_CHNL].BOX_Check, Chk_BOX_Function[bChannelNumber]);		
		return;		/* command is aborted.*/
	}
		
	if (EnclosureEnterState(pBox) == TRUE ) 
	{
		pStatus->NeedCheck = FALSE;


		/* Is disk replaced ? */
		bData = ReadIndexUchar(BaseAddress, iNDEXIDEDeviceHead);
		bData &= 0x0F;
		if ( bData == 0x0C ) {
			pStatus->NeedCheck = TRUE;
			gDrvConfig[bChannelNumber][0].DevFlag = DEV_NOTFOUND;
			EnclosureExitState(pBox);
			cam_SetStatusByPlug_unplug(bChannelNumber, 0, NEW_DRIVE);
			WriteIndexUchar(BaseAddress, iNDEXIDEDeviceHead, (U8)(bData&0xFB));	/* clear repd bit */
		}
		else if (( bData & 0x03 ) && (gDrvConfig[bChannelNumber][0].DevFlag & DEV_EXIST)) {
			pStatus->NeedCheck = TRUE;
			gDrvConfig[bChannelNumber][0].DevFlag = DEV_NOTFOUND;
			EnclosureExitState(pBox);
			cam_SetStatusByPlug_unplug(bChannelNumber, 0, NO_DRIVE);
		}
		else if ( !(gDrvConfig[bChannelNumber][0].DevFlag & DEV_EXIST) && ( bData == 0x08 ) ) {
			pStatus->NeedCheck = TRUE;
			EnclosureExitState(pBox);
			cam_SetStatusByPlug_unplug(bChannelNumber, 0, NEW_DRIVE);
		}
		
		/* Need to get box status ? */
		if ( pBox->BOXConfig & WITH_SENSOR) {
			if ( ( pStatus->RemPeriod == 0 )
		  	|| ((pStatus->RemPeriod = pStatus->RemPeriod-10) == 0) ) 
		  	{
				EnclosureExitState(pBox);
		 		Get_Enclosure_Datas(bChannelNumber);
			}
			else
				EnclosureExitState(pBox);
		}
		else
			EnclosureExitState(pBox);
			
		BOX_Timeout_TimerID[bChannelNumber] = camTimer(AdapterNumber,CAMTimeoutTimes[bChannelNumber/MAX_ATA_CHNL].BOX_Check, Chk_BOX_Function[bChannelNumber]);
		
	}
	else {
		gBOXStatus[bChannelNumber].PowerState = PowerOFF; /* can't enter state again due to power off */
		if ( gDrvConfig[bChannelNumber][0].DevFlag & DEV_EXIST ){
			gDrvConfig[bChannelNumber][0].DevFlag = DEV_NOTFOUND;
			cam_SetStatusByPlug_unplug(bChannelNumber, 0, NO_DRIVE);
		}		
		BOX_Timeout_TimerID[bChannelNumber] = camTimer(AdapterNumber,CAMTimeoutTimes[bChannelNumber/MAX_ATA_CHNL].BOX_Check, Chk_BOX_Function[bChannelNumber]);
	}
	return;
}

U8 CAM_Get_Enclosure_Status(U8 bChannelNumber, PENCLOSURE_STATUS pBoxStatus)
{
	PENCLOSURE_TYPE pBox;
	
	bChannelNumber=CAM_ATA_Chnl_InMAP(bChannelNumber);
	pBox =&gBOXConfig[bChannelNumber];
	
	if ((pBox->BOXType == SuperSwap) && (pBox->BOXConfig & MODE_SF )){
		*pBoxStatus = *(&gBOXStatus[bChannelNumber]);
					
		return(camSUCCESS);
	}
	else if ((pBox->BOXType != camNOBOX) && (pBox->BOXType & SATAEnclosure)) { 
		
		*pBoxStatus = *(&gBOXStatus[bChannelNumber]);
		return(camSUCCESS);
	}
	else
		return(camFAIL);
}

void Set_Box_LED(PATA_TASK pATATask)
{
	PTR_SIZE  bColor;
	U8	bData;
	U8	bChannelNumber = pATATask->bCh2Chip;
	PENCLOSURE_TYPE	pBox = &gBOXConfig[bChannelNumber];

	if( pATATask->ATAStatus & ATA_ABORT ) {
		return;		/* command is aborted.*/
	}
	
	bColor=(PTR_SIZE)pATATask->pTaskExtension;
	EnclosureEnterState(pBox);
	bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorCount);
	bData &= ~LED_MASK;
	bData |= ((U8)bColor & LED_MASK);
	WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorCount, bData );
	EnclosureExitState(pBox);
	
	return;
}

void CAM_ATA_CtrlLED(U8 bChannelNumber, U8 bColor)
{
	PATA_TASK pATATask = &boxLEDATATask[bChannelNumber];
	
	if ((gChnlConfig[bChannelNumber].bCh == camCHNL_NOT_FOUND) || (gBOXConfig[bChannelNumber].BOXType != SuperSwap))
		return;
	
	pATATask->bCh = CAM_ATA_Chnl_OutMAP(bChannelNumber);
	pATATask->bID = 0;
	pATATask->ATACmdFlag = SWAPBOX_CMD;
	switch(bColor) {
		case LED_Red:
			pATATask->pTaskExtension = (PVOID) (U32)(1);
			break;
		case LED_Green:
			pATATask->pTaskExtension = (PVOID) (U32)(2);
			break;
		case LED_Amber:
			pATATask->pTaskExtension = (PVOID) (U32)(3);
			break;
		default:
			return;	
	}	
	
	pATATask->callback = Set_Box_LED;	
	
	CAM_SubmitATA (pATATask);
	return;
}

/* this function only support for SuperSwap, not support for SuperSwap1100/SuperSwap4100/PSS10398 */
U8 CAM_Enclosure_PowerOn(U8 bChannelNumber)
{
	U8	bData;
	PENCLOSURE_TYPE	pBox;

	bChannelNumber=CAM_ATA_Chnl_InMAP(bChannelNumber);
	pBox = &gBOXConfig[bChannelNumber];
	
	if ((gChnlConfig[bChannelNumber].bCh == camCHNL_NOT_FOUND) || (gBOXConfig[bChannelNumber].BOXType != SuperSwap))
		return(camFAIL);
		
	/* at any time, if want to EnclosureEnterState or EnclosureExitState, must make sure that there is 
	     no command be running.*/
	stopATAModule(bChannelNumber);
	
	EnclosureEnterState(pBox);
	bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
	
	if ( pBox->BOXConfig & MODE_SF )
	{
		if (!(bData & bit(0)) && !(bData & bit(3))) {
			WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorCount, 0x07 );
			camStallExecution(1000);/* wait 10 seconds */
		}
		
		bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
		gBOXStatus[bChannelNumber].PowerState = ( bData & bit(3) )? PowerON : PowerOFF;
	}

	EnclosureExitState(pBox);

	return(camSUCCESS);
}

/* this function only support for SuperSwap, not support for SuperSwap1100/SuperSwap4100/PSS10398 */
U8 CAM_Enclosure_PowerOff(U8 bChannelNumber)
{
	U8	bData;
	PENCLOSURE_TYPE	pBox = &gBOXConfig[bChannelNumber];

	if ((gChnlConfig[bChannelNumber].bCh == camCHNL_NOT_FOUND) || (gBOXConfig[bChannelNumber].BOXType != SuperSwap))
		return(camFAIL);
		
	/* at any time, if want to EnclosureEnterState or EnclosureExitState, must make sure that there is 
	     no command be running.*/
	stopATAModule(bChannelNumber);
	
	EnclosureEnterState(pBox);
	bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
	
	if ( pBox->BOXConfig & MODE_SF )
	{
		bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
		bData &= ~bit(3);
		WriteIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDESectorCount, bData);
		gBOXStatus[bChannelNumber].PowerState =  PowerOFF;
	}
	EnclosureExitState(pBox);	
	return(camSUCCESS);
}

void Check_Enclosure_PowerState(U8 bChannelNumber)
{
	U8	bData;
	PENCLOSURE_TYPE	pBox = &gBOXConfig[bChannelNumber];
	
	if(gChnlConfig[bChannelNumber].ChnlType != CHNL_ATA)
		return;	
	
	if ( EnclosureEnterState(pBox)== TRUE ) {
		bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
	
		if ( pBox->BOXConfig & MODE_SF )
		{
			bData = ReadIndexUchar(ATABaseAddr[bChannelNumber], iNDEXIDEDeviceHead);
			gBOXStatus[bChannelNumber].PowerState = ( bData & bit(3) )? PowerON : PowerOFF;
		}

		EnclosureExitState(pBox);
	}
	else
		gBOXStatus[bChannelNumber].PowerState = PowerOFF; 
		
	return;
}

U8 EnclosureEnterState(PENCLOSURE_TYPE pBox)
{
	PTR_SIZE	BaseAddress = ATABaseAddr[pBox->BOXID & 0x0F];
	U8	bData, btmp=0;
	
	if ( pBox->BOXConfig & MODE_SF )
	{
		bData = ReadIndexUchar( BaseAddress, (iNDEXIDECICR+1));
		bData |= 0x01;	/* disable CSN */
		WriteIndexUchar( BaseAddress, (iNDEXIDECICR+1), bData);
		
		/* Is the box really in SF mode ? */
		bData = ReadIndexUchar(BaseAddress, iNDEXIDECylinderHigh);
		bData &= 0x0F;
		WriteIndexUchar( BaseAddress, iNDEXIDECylinderHigh, bData);
		bData = ReadIndexUchar(BaseAddress, iNDEXIDECylinderHigh);		
		if ( bData == 0x03 )
			return(TRUE);
			
		bData = ReadIndexUchar( BaseAddress, (iNDEXIDECICR+1));
		bData &= ~0x01;	/* re-enable CS */
		WriteIndexUchar( BaseAddress, (iNDEXIDECICR+1), bData);
		
		/* we need to set 199 to SF mode again */
	}
	
	/* try to enter state first time */
	bData=0xE0;
	WriteIndexUchar(BaseAddress, iNDEXIDEDeviceHead, bData);		/* to make sure it isn't in state 6 now */
	ReadIndexUchar(BaseAddress, iNDEXIDEStatus);
	bData = 0x80;
	WriteIndexUchar(BaseAddress, iNDEXIDESectorCount, bData );		/* make a maker */
	
	/* enter state */	
	bData=0xE0;
	WriteIndexUchar(BaseAddress, iNDEXIDEDeviceHead, bData );
	WriteIndexUchar(BaseAddress, iNDEXIDEDeviceHead, bData );
	WriteIndexUchar(BaseAddress, iNDEXIDEDeviceHead, bData );
	ReadIndexUchar(BaseAddress, iNDEXIDESectorCount);
	ReadIndexUchar(BaseAddress, iNDEXIDEDeviceHead);
	
	bData = ReadIndexUchar(BaseAddress, iNDEXIDESectorCount);
	if( bData & 0x80 )
		return(FALSE);		/* no box */
	
	btmp = bData | 0x04;	/* PWR bit of SF mode ! */

	/* check enclosure again */
	bData &= ~0x03;	
	bData |= 0x01;	/* red color  */
	WriteIndexUchar(BaseAddress, iNDEXIDESectorCount, bData );
	bData = ReadIndexUchar(BaseAddress, iNDEXIDESectorCount);
	if( (bData & 0x03) != 0x01 )
		return(FALSE);		/* no box */

	bData |= 0x03;	/* amber color  */
	WriteIndexUchar(BaseAddress, iNDEXIDESectorCount, bData );
	bData = 0;
	WriteIndexUchar(BaseAddress, iNDEXIDESectorNumber, bData );    /* for floating bus */
	bData = ReadIndexUchar(BaseAddress, iNDEXIDESectorCount);
	if( (bData & 0x03) != 0x03 )
		return(FALSE);		/* no box */

	bData = btmp & 0xDF; /* clear REP bit*/
	WriteIndexUchar(BaseAddress, iNDEXIDESectorCount, bData );	/* restore to orignal status */
	WriteIndexUchar(BaseAddress, iNDEXIDESectorNumber, 0x03 );	/* restore to orignal status */

	/* Is the box already in SF mode ? (after a warm boot )*/
	bData = ReadIndexUchar(BaseAddress, iNDEXIDECylinderHigh);
	if ( bData & 0x02 ) {
		pBox->BOXConfig = MODE_SF + WITH_SENSOR + WITH_FAN + WITH_KEY;
		pBox->BOXLEDConfig = WITH_STATUS_LED;
	
		pBox->BOXType = SuperSwap;
		pBox->NumofBay = 1;
		pBox->NumofFAN = 1;
		pBox->NumofSensor = 1;
		return(TRUE);
	}
	
	/* according hardware pins to set config */	
	bData = ReadIndexUchar(BaseAddress, iNDEXIDEDeviceHead);
	pBox->BOXType = SuperSwap;
	if (!(bData & bit(2))) {
		pBox->BOXConfig |= WITH_SENSOR;
		pBox->NumofSensor = 1;
	}
			
	if (!(bData & bit(3))) {
		pBox->BOXConfig |= WITH_FAN;
		pBox->NumofFAN = 1;
	}
			
	if (!(bData & bit(6)))
		pBox->BOXConfig |= WITH_KEY;	
				
	/* can the box be SF mode ? */
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderHigh, 0x03 );
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderHigh, 0x10 ); /* Index 1 of extened registers */
	bData = ReadIndexUchar(BaseAddress, iNDEXIDECylinderHigh);
	if(bData & 0x10) {
		pBox->BOXConfig |=  MODE_SF;		
		WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x00 ); /* set power on delay Time of SF mode to 0 */
		return(TRUE);
	}
	else
		return(FALSE);
	
}

void EnclosureExitState(PENCLOSURE_TYPE pBox)
{
	U8  bData;
	PTR_SIZE	BaseAddress = ATABaseAddr[pBox->BOXID & 0x0F];
	
	if ( pBox->BOXConfig & MODE_SF )
	{
		bData = ReadIndexUchar( BaseAddress, (iNDEXIDECICR+1));
		bData &= 0xFE;	/* enable CSN */
		WriteIndexUchar( BaseAddress, (iNDEXIDECICR+1), bData);
	}
	
	bData=0xE0;
	WriteIndexUchar(BaseAddress, iNDEXIDEDeviceHead, bData);
	ReadIndexUchar(BaseAddress, iNDEXIDEStatus);
	
	return;
}



/* write value into WinBond Register, input Data value and Register Index */
void WriteWReg(PTR_SIZE BaseAddress, U8 Index, U8 Data )
{
	U8	status;
	U16	i;
	
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderHigh, 0xB0);	/* write index and data */
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, Data);	/* index 0Bh */		
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, Index);	/* index 0Ch */	
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x5A);	/* index 0Dh */
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x03);	/* index 0Eh - write 3 bytes */
	for ( i = 0; i < 100; i++ ) {
		status = ReadIndexUchar(BaseAddress, iNDEXIDECylinderLow);		/* index 0Fh */
		if ( status & 0x03 )
			break;	/* Error or Done */
		camStallExecution(1);
	}
}

void ReadWReg(PTR_SIZE BaseAddress, U8 Index, PU8 Data )
{
	U8	status;
	U16	i;

	WriteIndexUchar(BaseAddress, iNDEXIDECylinderHigh, 0xC0);	/* write index */
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, Index);	/* index 0Ch */	
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x5A);	/* index 0Dh */
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x02);	/* index 0Eh - write 2 bytes */
	for ( i = 0; i < 100; i++ ) {
		status = ReadIndexUchar(BaseAddress, iNDEXIDECylinderLow);		/* index 0Fh */
		if ( status & 0x03 )
			break;	/* Error or Done */
		camStallExecution(1);
	}
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderHigh, 0xD0);	/* write index */	
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x5B);	/* index 0Dh */
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderLow, 0x11);	/* index 0Eh */
	for ( i = 0; i < 100; i++ ) {
		status = ReadIndexUchar(BaseAddress, iNDEXIDECylinderLow);		/* index 0Fh */
		if ( status & 0x03 )
			break;	/* Error or Done */
		camStallExecution(1);
	}
	WriteIndexUchar(BaseAddress, iNDEXIDECylinderHigh, 0xD0);	/* write index */	
	*Data = ReadIndexUchar(BaseAddress, iNDEXIDECylinderLow);
	
}


U16 ReadWinBondID_W83783S(PTR_SIZE BaseAddress )
{
    U8	low, high;

    /* set CR4E bit7=0 */
    WriteWReg(BaseAddress,0x4E, 0x00);
    /*read vendor ID low byte */
    ReadWReg(BaseAddress,0x4F, (PU8)&low );

    /* set CR4E bit7=1 */
    WriteWReg(BaseAddress,0x4E, 0x80 );
    /* read vendor ID high byte */
    ReadWReg(BaseAddress,0x4F, (PU8)&high );
    return ((high << 8 ) | low);

}

/* setup fan divisor to 010b, which gives a min RPM = 1.35e06/(255 * 2^2) = 1323
 * this is done by setting bit2 = Reg 5D, bit5 = 0
 *                         bit1 = Reg 47, bit5 = 1
 *                         bit0 = Reg 46, bit4 = 0
 */
void SetupFanDivisor(PTR_SIZE BaseAddress, U8 Divisor)
{
	U8 fd;
	
	ReadWReg(BaseAddress, 0x5D, (PU8)&fd);
	fd &= ~0x20;	/* bit2 of divisor is bit 5 of reg5D */
	fd |= (Divisor & 0x04 ) << 3;
	WriteWReg(BaseAddress, 0x5D, fd);
	ReadWReg(BaseAddress, 0x47, (PU8)&fd);
	fd &= ~0x30;	/* bit1,0 of divisor is bit 4, 5 of reg47 */
	fd |= (Divisor & 0x03 ) << 4;	/* bit1,0 of divisor is bit 4, 5 of reg47 */
	WriteWReg(BaseAddress, 0x47, fd);
	return;
}

U32 ReadFanStatus(PTR_SIZE BaseAddress )
{
    U8	fd, fd2;	/* FAN1DIV */
    U32	divisor;
    U8	fan1cnt;	/* FAN1COUNT */
    const U32 Dividend = 0x149970; /* 1.35 x 10^6 */
	U32	rpm=0;

    ReadWReg(BaseAddress,0x5D, (PU8)&fd);
    fd &= 0x20;		/* get bit5 - FAN1DIV bit2 */
    fd = fd >> 3;	/* move to bit2 */

    ReadWReg(BaseAddress,0x47, (PU8)&fd2);
    fd2 &= 0x30;	/* get bit5,4 - FAN1DIV bit1,0 */
    fd2 = fd2 >> 4;	/* move to bit1,0 */

    fd |= fd2;
    divisor = 1 << fd;

    ReadWReg(BaseAddress,0x28, (PU8)&fan1cnt);

	if (!fan1cnt * divisor)
        rpm = 0;
	else   
		rpm = Dividend / (fan1cnt * divisor);

	return rpm;
}

/* read Temp from WinBond, in deg C */
/*   this is temp1 of the IC */

U16 ReadTemperature(PTR_SIZE BaseAddress)
{
	U8  msb, lsb;
	U16 temp, mag;
	int sign;

	/* select Bank 1 */
    WriteWReg(BaseAddress, 0x4E, 0x01);
	/* read MSB */
	ReadWReg(BaseAddress, 0x50, (PU8)&msb);
	/* read LSB */
	ReadWReg(BaseAddress, 0x51, (PU8)&lsb);
    /* msb has 8 bits, lsb has 1 bit (temp1 = 9 bit value) */
    temp = ((U16)msb << 1) | ((U16)lsb >> 7);
    /* this value is 9 bit 2's complement */
    /* resolutin is 0.5 deg, range is -128 to 127.5 */
    sign = (temp & 0x100) ? -1 : 1;
    mag = (temp + 1) & 0x1ff;
    temp = mag * sign;
	/* restore to bank 0 */
	WriteWReg(BaseAddress, 0x4E, 0x00);

	return temp;
}

/* Read 5V Voltage Level. The LSB is 0.016 V=16 mV
 * this gives 8 bit = 4.096 V max voltage. The serial
 * resistor is used to drop the 5V level to 4.096V
*/
U32 ReadVoltage5V(PTR_SIZE BaseAddress)
{
    U8 vbase=16; /* 8-bit ADC has a 16mv LSB */
    U32 voltage;
    U8 regval;
    U8 SR1=34, SR2=50;	/* Vin is divided into R2/(R1+R2), to fit */
                          	/* with 0-4096. SR=Serial Resistor	*/

    ReadWReg(BaseAddress,0x23, (PU8)&regval);
    voltage = regval * vbase;
    voltage =  voltage  * (SR1 + SR2) / SR2;

   return voltage;
}

U32 ReadVoltage12V(PTR_SIZE BaseAddress)
{
    U8 vbase=16; /* 8-bit ADC has a 16mv LSB */
    U32 voltage;
    U8 regval;
    U8 R1=28, R2=10;		/* Vin is divided into R2/(R1+R2), to fit */
							/* with 0-4096 */

    ReadWReg(BaseAddress, 0x24, (PU8)&regval);
    voltage = regval * vbase;
    voltage = voltage * (R1 + R2)/R2;

    return voltage;
}



void Chk_BOX(U8 bChannelNumber)
{
	PATA_TASK	pATATask = &boxATATask[bChannelNumber];
	CAM_I2C_QUEUE	I2C_QUEUE;
	U8		Adapter_ID = bChannelNumber/MAX_ATA_CHNL;

	BOX_Timeout_TimerID[bChannelNumber]=ZERO_NULL;	
	
	if (gChnlConfig[bChannelNumber].ChnlType == CHNL_ATA ){
	
		pATATask->bCh = CAM_ATA_Chnl_OutMAP(bChannelNumber);
		pATATask->bID = 0;
		pATATask->ATACmdFlag = SWAPBOX_CMD;
		pATATask->callback = Get_Box_Status;	
		
		CAM_SubmitATA (pATATask);
	}
	else{ /* SATA BOX */
		I2C_QUEUE.Data_CNT = 0;
		I2C_QUEUE.b_ID = bChannelNumber;
		I2C_QUEUE.callback = Get_SataBox_Status;
		I2C_QUEUE.Data = 0;
		I2C_QUEUE.LED_Type = 0;
		CAM_SerialBusWriteDATA(Adapter_ID,&I2C_QUEUE);					
	}
	return;

}

#if (MAX_ADAPTER_NUM>0)
void Chk_BOX0(void)
{
	Chk_BOX(0x00);
	return;
}

void Chk_BOX1(void)
{
	Chk_BOX(0x01);
	return;
}

void Chk_BOX2(void)
{
	Chk_BOX(0x02);
	return;
}

void Chk_BOX3(void)
{
	Chk_BOX(0x03);
	return;
}
#endif
#if (MAX_ADAPTER_NUM>1)
void Chk_BOX4(void)
{
	Chk_BOX(0x04);
	return;
}

void Chk_BOX5(void)
{
	Chk_BOX(0x05);
	return;
}

void Chk_BOX6(void)
{
	Chk_BOX(0x06);
	return;
}

void Chk_BOX7(void)
{
	Chk_BOX(0x07);
	return;
}
#endif
#if (MAX_ADAPTER_NUM>2)
void Chk_BOX8(void)
{
	Chk_BOX(0x08);
	return;
}

void Chk_BOX9(void)
{
	Chk_BOX(0x09);
	return;
}

void Chk_BOXA(void)
{
	Chk_BOX(0x0A);
	return;
}

void Chk_BOXB(void)
{
	Chk_BOX(0x0B);
	return;
}
#endif
#if (MAX_ADAPTER_NUM>3)
void Chk_BOXC(void)
{
	Chk_BOX(0x0C);
	return;
}

void Chk_BOXD(void)
{
	Chk_BOX(0x0D);
	return;
}

void Chk_BOXE(void)
{
	Chk_BOX(0x0E);
	return;
}

void Chk_BOXF(void)
{
	Chk_BOX(0x0F);
	return;
}
#endif

void (*Chk_BOX_Function[])(void) = {

#if (MAX_ADAPTER_NUM>0)
  Chk_BOX0,
  Chk_BOX1,
  Chk_BOX2,
  Chk_BOX3,
#endif
#if (MAX_ADAPTER_NUM>1)
  Chk_BOX4,
  Chk_BOX5,
  Chk_BOX6,
  Chk_BOX7,
#endif
#if (MAX_ADAPTER_NUM>2)
  Chk_BOX8,
  Chk_BOX9,
  Chk_BOXA,
  Chk_BOXB,
#endif  
#if (MAX_ADAPTER_NUM>3)
  Chk_BOXC,
  Chk_BOXD,
  Chk_BOXE,
  Chk_BOXF,  
#endif  
};


U16 ReadWinBondID_W83782D(U8 Adapter_ID,U8 bID)
{
	U8	low, high;
	
	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);
	
	/* set CR4E bit7=0 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x004E50,3);
	/*read vendor ID low byte */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4F50,2);
	low=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set CR4E bit7=1 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x804E50,3);
	/*read vendor ID low byte */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4F50,2);
	high=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
		
	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);
	
/*	I2CBusy[Adapter_ID] = FALSE;
*	CAM_StartI2C(Adapter_ID);
*/
		
    return ((high << 8 ) | low);
}

void InternalI2CReturn(PCAM_I2C_QUEUE pI2C_QUEUE)
{
	return;
}

void CAM_SBOX_ActiveLED(U8 Adapter_ID,U8 bID, PCAM_SBOX_LED pLED,PVOID callBack)
{
	U32 data=0;
	CAM_I2C_QUEUE I2C_QUEUE;
	
	if (Adapter_ID > (MAX_ADAPTER_NUM -1))
		return;
			
	/* is SATA BOX exist */
	if (!(SBOXType[Adapter_ID][bID].BOXConfig & SBOXEXIST))
		return;	
	
	if (pLED->LED.ActiveLed !=LEDNOCHANGE) /* need change */
	{
		I2C_QUEUE.Data_CNT = 1;
		I2C_QUEUE.b_ID = Adapter_ID*MAX_ATA_CHNL+bID;
		I2C_QUEUE.callback = (void (*)(PCAM_I2C_QUEUE))callBack;
	
		if (pLED->LED.ActiveLed==LEDOFF)
			I2C_QUEUE.Data = (0x07+0x10*bID)<<1;
		else if (pLED->LED.ActiveLed==LEDON)
			I2C_QUEUE.Data = (0x06+0x10*bID)<<1;
		else
			return;
		I2C_QUEUE.LED_Type=ActiveLED;
		CAM_SerialBusWriteDATA(Adapter_ID,&I2C_QUEUE);
	}
	
	if (pLED->Flash !=LEDNOCHANGE) /* need change */
	{
		switch(pLED->Flash) {
			case NOFLASH:
				data=0;
				break;
			case SLOWFLASH:
				data=2;
				break;
			case FASTFLASH:
				data=3;
				break;
			default:
				return;
		}
			
		I2C_QUEUE.Data_CNT = 2;
		I2C_QUEUE.b_ID = Adapter_ID*MAX_ATA_CHNL+bID;
		I2C_QUEUE.callback =  (void (*)(PCAM_I2C_QUEUE))callBack;
			
		data = data << 8;
		I2C_QUEUE.Data = data | ((0x09+0x10*bID)<<1);
		I2C_QUEUE.LED_Type=ActiveLED;
		CAM_SerialBusWriteDATA(Adapter_ID,&I2C_QUEUE);
	}
	return;
}

void CAM_SBOX_StatusLED(U8 Adapter_ID,U8 bID, PCAM_SBOX_LED pLED,PVOID callBack)
{
	U32 data=0;
	CAM_I2C_QUEUE I2C_QUEUE;
	U8 double_change=FALSE;
	
	if (Adapter_ID > (MAX_ADAPTER_NUM -1))
		return;

	/* is SATA BOX exist */
	if (!(SBOXType[Adapter_ID][bID].BOXConfig & SBOXEXIST))
		return;

	if ((pLED->LED.StatusLed !=LEDNOCHANGE) && (pLED->Flash !=LEDNOCHANGE))
		double_change=TRUE;
		
	if (pLED->LED.StatusLed !=LEDNOCHANGE) /* need change */
	{
		switch(pLED->LED.StatusLed) {
			case LED_Red:
				data=3;
				break;
			case LED_Green:
				data=4;
				break;
			case LED_Amber:
				data=5;
				break;
			case LEDOFF:
				data=0x0C;
				break;				
			default:
				return;	
		}
		I2C_QUEUE.Data_CNT = 1;
		I2C_QUEUE.b_ID = Adapter_ID*MAX_ATA_CHNL+bID;

		if (double_change)
			I2C_QUEUE.callback = InternalI2CReturn; /* callback do nothing */
		else
			I2C_QUEUE.callback =  (void (*)(PCAM_I2C_QUEUE))callBack;
			
		I2C_QUEUE.Data = (data+0x10*bID)<<1;
		I2C_QUEUE.LED_Type=StatusLED;
		CAM_SerialBusWriteDATA(Adapter_ID,&I2C_QUEUE);	
	}
	
	data = 0;
	if (pLED->Flash !=LEDNOCHANGE) /* need change */
	{
		switch(pLED->Flash) {
			case NOFLASH:
				data=0;
				break;
			case SLOWFLASH:
				data=2;
				break;
			case FASTFLASH:
				data=3;
				break;
			default:
				return;	
		}

		I2C_QUEUE.Data_CNT = 2;
		I2C_QUEUE.b_ID = Adapter_ID*MAX_ATA_CHNL+bID;
		I2C_QUEUE.callback =  (void (*)(PCAM_I2C_QUEUE))callBack;
			
		data |= 0x10;
		data = data << 8;			
		I2C_QUEUE.Data = data | ((0x09+0x10*bID)<<1);
		I2C_QUEUE.LED_Type=StatusLED;
		CAM_SerialBusWriteDATA(Adapter_ID,&I2C_QUEUE);			
	}
	return;
}

void CAM_SBOX_GlobalLED(U8 Adapter_ID,U8 bID, PCAM_SBOX_LED pLED,PVOID callBack)
{
	U32 data=0;
	CAM_I2C_QUEUE I2C_QUEUE;
	
	if (Adapter_ID > (MAX_ADAPTER_NUM -1))
		return;

	/* is SATA BOX exist */
	if (!(SBOXType[Adapter_ID][bID].BOXConfig & SBOXEXIST))
		return;
	
	data =0; /* LEDOFF */

	if (pLED->LED.GlobalLed ==LEDNOCHANGE) /* no change */
		pLED->LED.GlobalLed=SBOXType[Adapter_ID][bID].LEDColor.GlobalLEDColor;
	
	if (pLED->Flash ==LEDNOCHANGE) /* no change */
		pLED->Flash=SBOXType[Adapter_ID][bID].Flash.GlobalFlash;
					
	if ( pLED->LED.GlobalLed != LEDOFF)
	{
		switch(pLED->LED.GlobalLed) {
			case LED_Red:
				switch(pLED->Flash) {
					case NOFLASH:
						data=0x10;
						break;
					case SLOWFLASH:
						data=0x20;
						break;
					case FASTFLASH:
						data=0x30;
						break;
					default:
						return;	
				}			
				break;
			case LED_Green:
				switch(pLED->Flash) {
					case NOFLASH:
						data=0x40;
						break;
					case SLOWFLASH:
						data=0x80;
						break;
					case FASTFLASH:
						data=0xC0;
						break;
					default:
						return;	
				}			
				break;
			case LED_Amber:
				switch(pLED->Flash) {
					case NOFLASH:
						data=0x50;
						break;
					case SLOWFLASH:
						data=0xA0;
						break;
					case FASTFLASH:
						data=0xF0;
						break;
					default:
						return;	
				}			
				break;
			default:
				return;	
		}
	}
				
	I2C_QUEUE.Data_CNT = 2;
	I2C_QUEUE.b_ID = Adapter_ID*MAX_ATA_CHNL+bID;
	I2C_QUEUE.callback =  (void (*)(PCAM_I2C_QUEUE))callBack;
		
	data = data << 8;			
	I2C_QUEUE.Data = data | ((0x0B+0x10*bID)<<1);
	I2C_QUEUE.LED_Type=GlobalLED;
	CAM_SerialBusWriteDATA(Adapter_ID,&I2C_QUEUE);		
	return;
}

void HDAccess_LEDI2CReturn(PCAM_I2C_QUEUE pI2C_QUEUE)
{
	U8 b_ID = pI2C_QUEUE->b_ID;
	U8 Adapter_ID = b_ID/MAX_ATA_CHNL;
	U8 bID = b_ID % MAX_ATA_CHNL;
	CAM_SBOX_LED LED0;
	
	switch (pI2C_QUEUE->LED_Type) {
		case StatusLED:
			if(HDAccess_StatusLED_Buffer[b_ID][0].NeedRetryFlag == TRUE){
				if(HDAccess_StatusLED_Buffer[b_ID][0].RetryCount < I2C_DELAY_COMMAND_COUNT){
					HDAccess_StatusLED_Buffer[b_ID][0].RetryCount++;

					LED0.LED.StatusLed = HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.LED.StatusLed;
					LED0.Flash = HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.Flash;
					CAM_SBOX_StatusLED(Adapter_ID,bID,&LED0,HDAccess_LEDI2CReturn);		
					
					break;
				}
				HDAccess_StatusLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
			}	

			if (HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag == TRUE) {
				HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag = FALSE;
				HDAccess_StatusLED_Buffer[b_ID][0].UsedFlag = TRUE;
				
				HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.LED.StatusLed=HDAccess_StatusLED_Buffer[b_ID][1].Buffer_LED.LED.StatusLed;
				HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.Flash=HDAccess_StatusLED_Buffer[b_ID][1].Buffer_LED.Flash;
				HDAccess_StatusLED_Buffer[b_ID][0].b_ID = HDAccess_StatusLED_Buffer[b_ID][1].b_ID;
		
				LED0.LED.StatusLed = SBOXType[b_ID/MAX_ATA_CHNL][b_ID%MAX_ATA_CHNL].LEDColor.StatusLEDColor;
				LED0.Flash = HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.Flash;		

				CAM_SBOX_StatusLED(Adapter_ID,bID,&LED0,HDAccess_LEDI2CReturn);		
			}
			else
				HDAccess_StatusLED_Buffer[b_ID][0].UsedFlag = FALSE;		
			break;		
		case ActiveLED:
			if(HDAccess_ActiveLED_Buffer[b_ID][0].NeedRetryFlag == TRUE){
				if(HDAccess_ActiveLED_Buffer[b_ID][0].RetryCount < I2C_DELAY_COMMAND_COUNT){
					HDAccess_ActiveLED_Buffer[b_ID][0].RetryCount++;

					LED0.LED.ActiveLed = HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.LED.ActiveLed;
					LED0.Flash = HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.Flash;
					CAM_SBOX_ActiveLED(Adapter_ID,bID,&LED0,HDAccess_LEDI2CReturn);		
					
					break;
				}
				HDAccess_ActiveLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
			}	
			
			if (HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag == TRUE) {
				HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag = FALSE;
				HDAccess_ActiveLED_Buffer[b_ID][0].UsedFlag = TRUE;
				
				HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.LED.ActiveLed=HDAccess_ActiveLED_Buffer[b_ID][1].Buffer_LED.LED.ActiveLed;
				HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.Flash=HDAccess_ActiveLED_Buffer[b_ID][1].Buffer_LED.Flash;
				HDAccess_ActiveLED_Buffer[b_ID][0].b_ID = HDAccess_ActiveLED_Buffer[b_ID][1].b_ID;
		
				LED0.LED.ActiveLed = HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.LED.ActiveLed;
				LED0.Flash = HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.Flash;		

				CAM_SBOX_ActiveLED(Adapter_ID,bID,&LED0,HDAccess_LEDI2CReturn);		
			}
			else
				HDAccess_ActiveLED_Buffer[b_ID][0].UsedFlag = FALSE;
			break;
		case GlobalLED:
			if(HDAccess_GlobalLED_Buffer[b_ID][0].NeedRetryFlag == TRUE){
				if(HDAccess_GlobalLED_Buffer[b_ID][0].RetryCount < I2C_DELAY_COMMAND_COUNT){
					HDAccess_GlobalLED_Buffer[b_ID][0].RetryCount++;

					LED0.LED.GlobalLed = HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.LED.GlobalLed;
					LED0.Flash = HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.Flash;
					CAM_SBOX_GlobalLED(Adapter_ID,bID,&LED0,HDAccess_LEDI2CReturn);		
					
					break;
				}
				HDAccess_GlobalLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
			}	

			if (HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag == TRUE) {
				HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag = FALSE;
				HDAccess_GlobalLED_Buffer[b_ID][0].UsedFlag = TRUE;
				
				HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.LED.GlobalLed=HDAccess_GlobalLED_Buffer[b_ID][1].Buffer_LED.LED.GlobalLed;
				HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.Flash=HDAccess_GlobalLED_Buffer[b_ID][1].Buffer_LED.Flash;
				HDAccess_GlobalLED_Buffer[b_ID][0].b_ID = HDAccess_GlobalLED_Buffer[b_ID][1].b_ID;
		
				LED0.LED.GlobalLed = SBOXType[b_ID/MAX_ATA_CHNL][b_ID%MAX_ATA_CHNL].LEDColor.GlobalLEDColor;
				LED0.Flash = HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.Flash;		
				
				CAM_SBOX_GlobalLED(Adapter_ID,bID,&LED0,HDAccess_LEDI2CReturn);
			}
			else
				HDAccess_GlobalLED_Buffer[b_ID][0].UsedFlag = FALSE;
			break;		
	}
	return;
}

U8 PutHDAccess_LED(U8 LED_Type,U8 b_ID,PCAM_SBOX_LED pLED0)
{
	U8 Adapter_ID = b_ID/MAX_ATA_CHNL;
	U8 bID = b_ID % MAX_ATA_CHNL;
	
	switch (LED_Type) {
		case StatusLED:
			/* check is need put into buffer */
			if (pLED0->Flash==LEDNOCHANGE) { /* flash no change */
				/* check is equal to current led state */
				if ( ((pLED0->LED.StatusLed == LEDOFF) && (HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.LED.StatusLed==LEDOFF))
					|| ((pLED0->LED.StatusLed != LEDOFF) && (HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.LED.StatusLed!=LEDOFF)) )
				{
					if (HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag == TRUE) {/* equal to current led state, cancel next change state */
						HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag = FALSE;
						HDAccess_StatusLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
					}
					break;
				}
				else {
					if (HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag == TRUE)
						break;
				}				
				pLED0->Flash = HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.Flash;
			}
			else { /* (pLED0->LED.StatusLed==LEDNOCHANGE) */  /* StatusLed no change */
				/* check is equal to current led state */
				if (pLED0->Flash == HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.Flash) {
					if (HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag == TRUE){ /* equal to current led state, cancel next change state */
						HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag = FALSE;
						HDAccess_StatusLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
					}
					break;
				}
				else {
					if (HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag == TRUE)
						break;
				}
				pLED0->LED.StatusLed = SBOXType[b_ID/MAX_ATA_CHNL][b_ID%MAX_ATA_CHNL].LEDColor.StatusLEDColor;
			}
		
			if (HDAccess_StatusLED_Buffer[b_ID][0].UsedFlag == FALSE) {
				HDAccess_StatusLED_Buffer[b_ID][0].UsedFlag = TRUE;
				
				HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.LED.StatusLed=pLED0->LED.StatusLed;
				HDAccess_StatusLED_Buffer[b_ID][0].Buffer_LED.Flash=pLED0->Flash;
				HDAccess_StatusLED_Buffer[b_ID][0].b_ID = b_ID;

				/* HD access ON case */
				if( ((pLED0->Flash != LEDNOCHANGE) && (pLED0->Flash != NOFLASH)) ||
					((pLED0->Flash == LEDNOCHANGE) && (pLED0->LED.StatusLed != LEDOFF))){
					HDAccess_StatusLED_Buffer[b_ID][0].RetryCount = 0;
					HDAccess_StatusLED_Buffer[b_ID][0].NeedRetryFlag = TRUE;
				}
				
				CAM_SBOX_StatusLED(Adapter_ID,bID,pLED0,HDAccess_LEDI2CReturn);
			}
			else if (HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag == FALSE) {
				HDAccess_StatusLED_Buffer[b_ID][1].UsedFlag = TRUE;
				
				HDAccess_StatusLED_Buffer[b_ID][1].Buffer_LED.LED.StatusLed=pLED0->LED.StatusLed;
				HDAccess_StatusLED_Buffer[b_ID][1].Buffer_LED.Flash=pLED0->Flash;
				HDAccess_StatusLED_Buffer[b_ID][1].b_ID = b_ID;				
			}
			else
				return camFAIL;
			break;
		case ActiveLED:
			/* check is need put into buffer */
			if (pLED0->Flash==LEDNOCHANGE) { /* flash no change */
				/* check is equal to current led state */
				if (pLED0->LED.ActiveLed == HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.LED.ActiveLed) {
					if (HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag == TRUE){ /* equal to current led state, cancel next change state */
						HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag = FALSE;
						HDAccess_ActiveLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
					}
					break;
				}
				else {
					if (HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag == TRUE)
						break;
				}				
				
			}
			else { /* (pLED0->LED.ActiveLed==LEDNOCHANGE) */  /* ActiveLed no change */
				/* check is equal to current led state */
				if (pLED0->Flash == HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.Flash) {
					if (HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag == TRUE){ /* equal to current led state, cancel next change state */
						HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag = FALSE;
						HDAccess_ActiveLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
					}
					break;
				}
				else {
					if (HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag == TRUE)
						break;
				}
			}
		
			if (HDAccess_ActiveLED_Buffer[b_ID][0].UsedFlag == FALSE) {
				HDAccess_ActiveLED_Buffer[b_ID][0].UsedFlag = TRUE;
				
				HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.LED.ActiveLed=pLED0->LED.ActiveLed;
				HDAccess_ActiveLED_Buffer[b_ID][0].Buffer_LED.Flash=pLED0->Flash;
				HDAccess_ActiveLED_Buffer[b_ID][0].b_ID = b_ID;

				/* HD access ON case */
				if( ((pLED0->Flash != LEDNOCHANGE) && (pLED0->Flash != NOFLASH)) ||
					((pLED0->Flash == LEDNOCHANGE) && (pLED0->LED.ActiveLed != LEDOFF))){
					HDAccess_ActiveLED_Buffer[b_ID][0].RetryCount = 0;
					HDAccess_ActiveLED_Buffer[b_ID][0].NeedRetryFlag = TRUE;
				}

				CAM_SBOX_ActiveLED(Adapter_ID,bID,pLED0,HDAccess_LEDI2CReturn);
			}
			else if (HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag == FALSE) {
				HDAccess_ActiveLED_Buffer[b_ID][1].UsedFlag = TRUE;
				
				HDAccess_ActiveLED_Buffer[b_ID][1].Buffer_LED.LED.ActiveLed=pLED0->LED.ActiveLed;
				HDAccess_ActiveLED_Buffer[b_ID][1].Buffer_LED.Flash=pLED0->Flash;
				HDAccess_ActiveLED_Buffer[b_ID][1].b_ID = b_ID;				
			}
			else
				return camFAIL;
			break;
		case GlobalLED:
			/* check is need put into buffer */
			if (pLED0->Flash==LEDNOCHANGE) { /* flash no change */
				/* check is equal to current led state */
				if ( ((pLED0->LED.GlobalLed == LEDOFF) && (HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.LED.GlobalLed==LEDOFF))
					|| ((pLED0->LED.GlobalLed != LEDOFF) && (HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.LED.GlobalLed!=LEDOFF)) )
				{
					if (HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag == TRUE){ /* equal to current led state, cancel next change state */
						HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag = FALSE;
						HDAccess_GlobalLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
					}
					break;
				}
				else {
					if (HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag == TRUE)
						break;
				}				
				pLED0->Flash = HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.Flash;
			}
			else { /* (pLED0->LED.GlobalLed==LEDNOCHANGE) */  /* GlobalLed no change */
				/* check is equal to current led state */
				if (pLED0->Flash == HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.Flash) {
					if (HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag == TRUE){ /* equal to current led state, cancel next change state */
						HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag = FALSE;
						HDAccess_GlobalLED_Buffer[b_ID][0].NeedRetryFlag = FALSE;
					}
					break;
				}
				else {
					if (HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag == TRUE)
						break;
				}
				pLED0->LED.GlobalLed = SBOXType[b_ID/MAX_ATA_CHNL][b_ID%MAX_ATA_CHNL].LEDColor.GlobalLEDColor;
			}
		
			if (HDAccess_GlobalLED_Buffer[b_ID][0].UsedFlag == FALSE) {
				HDAccess_GlobalLED_Buffer[b_ID][0].UsedFlag = TRUE;
				
				HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.LED.GlobalLed=pLED0->LED.GlobalLed;
				HDAccess_GlobalLED_Buffer[b_ID][0].Buffer_LED.Flash=pLED0->Flash;
				HDAccess_GlobalLED_Buffer[b_ID][0].b_ID = b_ID;

				/* HD access ON case */
				if( ((pLED0->Flash != LEDNOCHANGE) && (pLED0->Flash != NOFLASH)) ||
					((pLED0->Flash == LEDNOCHANGE) && (pLED0->LED.GlobalLed != LEDOFF))){
					HDAccess_GlobalLED_Buffer[b_ID][0].RetryCount = 0;
					HDAccess_GlobalLED_Buffer[b_ID][0].NeedRetryFlag = TRUE;
				}

				CAM_SBOX_GlobalLED(Adapter_ID,bID,pLED0,HDAccess_LEDI2CReturn);
			}
			else if (HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag == FALSE) {
				HDAccess_GlobalLED_Buffer[b_ID][1].UsedFlag = TRUE;
				
				HDAccess_GlobalLED_Buffer[b_ID][1].Buffer_LED.LED.GlobalLed=pLED0->LED.GlobalLed;
				HDAccess_GlobalLED_Buffer[b_ID][1].Buffer_LED.Flash=pLED0->Flash;
				HDAccess_GlobalLED_Buffer[b_ID][1].b_ID = b_ID;				
			}
			else
				return camFAIL;
			break;		
	}
	return camSUCCESS;
}

void CAM_SBOX_HDAccess_LED(U8 bChannelNumber , U8 Active)
{

	CAM_SBOX_LED sboxLED;
	
	U8 Adapter_ID = bChannelNumber/MAX_ATA_CHNL;
	U8 bID;
	
	if (Adapter_ID > (MAX_ADAPTER_NUM -1))
		return;
		
	bID=ChnlMapToSBOXChnl[bChannelNumber];
	/* is SATA BOX exist */
	if (!(SBOXType[Adapter_ID][bID].BOXConfig & SBOXEXIST))
		return;

	/* StatusLED */
	if (CAM_SBOX_HDAccess_LEDs[Adapter_ID] & StatusLED)
	{
		if (CAM_SBOX_HDAccessLED[bChannelNumber].Status_LED.LED_OnOff==LEDOFF) {
			if (Active ==HDActiveOFF )
				sboxLED.LED.StatusLed=LEDOFF;
			else /* HDActiveON */
				sboxLED.LED.StatusLed=LEDON;
			sboxLED.Flash=LEDNOCHANGE;
		}
		else { /* LEDON */
			sboxLED.LED.StatusLed=LEDNOCHANGE;
			if (Active ==HDActiveOFF )
				sboxLED.Flash=NOFLASH;
			else /* HDActiveON */
				sboxLED.Flash=CAM_SBOX_HDAccessLED[bChannelNumber].Status_LED.Flash;
		}
	
		PutHDAccess_LED(StatusLED,(U8)(Adapter_ID*MAX_ATA_CHNL+bID),&sboxLED);
	}

	/* ActiveLED */		
	if (CAM_SBOX_HDAccess_LEDs[Adapter_ID] & ActiveLED )
	{
		if (CAM_SBOX_HDAccessLED[bChannelNumber].Active_LED.LED_OnOff==LEDOFF) {
			if (Active ==HDActiveOFF )
				sboxLED.LED.ActiveLed=LEDOFF;
			else /* HDActiveON */
				sboxLED.LED.ActiveLed=LEDON;
			sboxLED.Flash=LEDNOCHANGE;
		}
		else { /* LEDON */
			sboxLED.LED.ActiveLed=LEDNOCHANGE;
			if (Active ==HDActiveOFF )
				sboxLED.Flash=NOFLASH;
			else /* HDActiveON */
				sboxLED.Flash=CAM_SBOX_HDAccessLED[bChannelNumber].Active_LED.Flash;
		}
	
		PutHDAccess_LED(ActiveLED,(U8)(Adapter_ID*MAX_ATA_CHNL+bID),&sboxLED);
	}

	/* GlobalLED */
	if (CAM_SBOX_HDAccess_LEDs[Adapter_ID] & GlobalLED )
	{
		if (CAM_SBOX_HDAccessLED[bChannelNumber].Global_LED.LED_OnOff==LEDOFF) {
			if (Active ==HDActiveOFF )
				sboxLED.LED.ActiveLed=LEDOFF;
			else /* HDActiveON */
				sboxLED.LED.ActiveLed=LEDON;
			sboxLED.Flash=LEDNOCHANGE;
		}
		else { /* LEDON */
			sboxLED.LED.ActiveLed=LEDNOCHANGE;
			if (Active ==HDActiveOFF )
				sboxLED.Flash=NOFLASH;
			else /* HDActiveON */
				sboxLED.Flash=CAM_SBOX_HDAccessLED[bChannelNumber].Global_LED.Flash;
		}
		PutHDAccess_LED(GlobalLED,(U8)(Adapter_ID*MAX_ATA_CHNL+bID),&sboxLED);
	}
	
		
	return;
}

/* must set LED_Type before CAM_Init */
/* LED_Types : (ActiveLED or StatusLED)/GlobalLED */
U8 CAM_Set_HDAccess_LED_All(U8 Adapter_ID, U8 LED_Types,PCAM_HDAccess_LED_All pHDAccess_LED_All)
{
	U8 i=0;
	
	/* LED_TYPE had set */
	if (AllHDAccessSetFlag[Adapter_ID] == TRUE) 
		return (camFAIL);

	/* CAM_InitSATAEnclosure been called */
	if (AdapterInitFlag[Adapter_ID] == TRUE)
		return (camFAIL);
				
	/* check parameters */
	if ( LED_Types & (~(ActiveLED|StatusLED|GlobalLED)) )
		return (camFAIL);
		
	/* only support flash ActiveLED or StatusLED */
	if ( (LED_Types & ActiveLED) && (LED_Types & StatusLED) )
		return (camFAIL);		
		
	if (LED_Types&StatusLED) {
		if ( (pHDAccess_LED_All->Status_LED.Flash!=NOFLASH) && (pHDAccess_LED_All->Status_LED.Flash!=SLOWFLASH) && (pHDAccess_LED_All->Status_LED.Flash!=FASTFLASH) )
			return (camFAIL);
		if ( (pHDAccess_LED_All->Status_LED.LED_OnOff!=LEDON) && (pHDAccess_LED_All->Status_LED.LED_OnOff!=LEDOFF) )
			return (camFAIL);
	}
	
	if (LED_Types&ActiveLED) {
		if ( (pHDAccess_LED_All->Active_LED.Flash!=NOFLASH) && (pHDAccess_LED_All->Active_LED.Flash!=SLOWFLASH) && (pHDAccess_LED_All->Active_LED.Flash!=FASTFLASH) )
			return (camFAIL);
		if ( (pHDAccess_LED_All->Active_LED.LED_OnOff!=LEDON) && (pHDAccess_LED_All->Active_LED.LED_OnOff!=LEDOFF) )
			return (camFAIL);			
	}	
	
	if (LED_Types&GlobalLED) {
		if ( (pHDAccess_LED_All->Global_LED.Flash!=NOFLASH) && (pHDAccess_LED_All->Global_LED.Flash!=SLOWFLASH) && (pHDAccess_LED_All->Global_LED.Flash!=FASTFLASH) )
			return (camFAIL);
		if ( (pHDAccess_LED_All->Global_LED.LED_OnOff!=LEDON) && (pHDAccess_LED_All->Global_LED.LED_OnOff!=LEDOFF) )
			return (camFAIL);			
	}
	
	CAM_SBOX_HDAccess_LEDs[Adapter_ID]=LED_Types;

	for( i = (Adapter_ID*MAX_ATA_CHNL); i < ((Adapter_ID+1)*MAX_ATA_CHNL); i++ ) {	
		if (LED_Types&StatusLED)
			CAM_SBOX_HDAccessLED[i].Status_LED.Flash = pHDAccess_LED_All->Status_LED.Flash;
		if (LED_Types&ActiveLED)
			CAM_SBOX_HDAccessLED[i].Active_LED.Flash = pHDAccess_LED_All->Active_LED.Flash;
		if (LED_Types&GlobalLED)
			CAM_SBOX_HDAccessLED[i].Global_LED.Flash = pHDAccess_LED_All->Global_LED.Flash;
	}
	
	for( i = (Adapter_ID*MAX_ATA_CHNL); i < ((Adapter_ID+1)*MAX_ATA_CHNL); i++ ) {	
		if (LED_Types&StatusLED)
			CAM_SBOX_HDAccessLED[i].Status_LED.LED_OnOff = pHDAccess_LED_All->Status_LED.LED_OnOff;
		if (LED_Types&ActiveLED)
			CAM_SBOX_HDAccessLED[i].Active_LED.LED_OnOff = pHDAccess_LED_All->Active_LED.LED_OnOff;
		if (LED_Types&GlobalLED)
			CAM_SBOX_HDAccessLED[i].Global_LED.LED_OnOff = pHDAccess_LED_All->Global_LED.LED_OnOff;
	}
		
	AllHDAccessSetFlag[Adapter_ID] = TRUE;
	return (camSUCCESS);
}

U8 CAM_Set_HDAccess_LED(U8 bChannelNumber,U8 LED_Type,U8 Flash)
{
	CAM_Ctrl_LED Ctrl_LED;
	
	if (bChannelNumber >= MAX_ATA_MODULE)
		return (camCHNL_NOT_FOUND);

	if ( (LED_Type!=ActiveLED) && (LED_Type!=StatusLED) && (LED_Type!=GlobalLED) )
		return (camFAIL);
					
	if ( (Flash!=NOFLASH) && (Flash!=SLOWFLASH) && (Flash!=FASTFLASH) )
		return (camFAIL);

	bChannelNumber=CAM_ATA_Chnl_InMAP(bChannelNumber);
	
	switch (LED_Type) {	
		case StatusLED:
			CAM_SBOX_HDAccessLED[bChannelNumber].Status_LED.Flash = Flash;
			if (CAM_SBOX_HDAccessLED[bChannelNumber].Status_LED.LED_OnOff==LEDOFF) {
				Ctrl_LED.LED.StatusLed=LEDNOCHANGE;
				Ctrl_LED.Flash=Flash;
				Ctrl_LED.LED_Type=StatusLED;
				CAM_CtrlLED(CAM_ATA_Chnl_OutMAP(bChannelNumber), &Ctrl_LED);				
			}
			break;		
		case ActiveLED:
			CAM_SBOX_HDAccessLED[bChannelNumber].Active_LED.Flash = Flash;
			if (CAM_SBOX_HDAccessLED[bChannelNumber].Active_LED.LED_OnOff==LEDOFF) {
				Ctrl_LED.LED.ActiveLed=LEDNOCHANGE;
				Ctrl_LED.Flash=Flash;
				Ctrl_LED.LED_Type=ActiveLED;
				CAM_CtrlLED(CAM_ATA_Chnl_OutMAP(bChannelNumber), &Ctrl_LED);				
			}
			break;			
		case GlobalLED:
			CAM_SBOX_HDAccessLED[bChannelNumber].Global_LED.Flash = Flash;
			if (CAM_SBOX_HDAccessLED[bChannelNumber].Global_LED.LED_OnOff==LEDOFF) {
				Ctrl_LED.LED.GlobalLed=LEDNOCHANGE;
				Ctrl_LED.Flash=Flash;
				Ctrl_LED.LED_Type=GlobalLED;
				CAM_CtrlLED(CAM_ATA_Chnl_OutMAP(bChannelNumber), &Ctrl_LED);				
			}			
			break;
	}
	return (camSUCCESS);
}

void CAM_CtrlLED(U8 bChannelNumber, PCAM_Ctrl_LED pCtrl_LED)
{
	U8 Adapter_ID;
	
	bChannelNumber=CAM_ATA_Chnl_InMAP(bChannelNumber);
	
	if (gChnlConfig[bChannelNumber].ChnlType == CHNL_ATA ) {
		if(pCtrl_LED->LED_Type == StatusLED)
			CAM_ATA_CtrlLED(bChannelNumber, pCtrl_LED->LED.StatusLed);
		return;
	}
	
	if ((gBOXConfig[bChannelNumber].BOXType == camNOBOX) || (!(gBOXConfig[bChannelNumber].BOXType & SATAEnclosure)) )
		return;
	
	Adapter_ID = bChannelNumber/MAX_ATA_CHNL;
	
	switch (pCtrl_LED->LED_Type) {
		case ActiveLED:
			CAM_SBOX_ActiveLED(Adapter_ID,ChnlMapToSBOXChnl[bChannelNumber], (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
			break;
		case StatusLED:
			CAM_SBOX_StatusLED(Adapter_ID,ChnlMapToSBOXChnl[bChannelNumber], (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
			if(pCtrl_LED->LED.StatusLed != LEDNOCHANGE)
				SBOXType[Adapter_ID][ChnlMapToSBOXChnl[bChannelNumber]].LEDColor.StatusLEDColor= pCtrl_LED->LED.StatusLed;
			break;
		case GlobalLED:
			CAM_SBOX_GlobalLED(Adapter_ID,ChnlMapToSBOXChnl[bChannelNumber], (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
			if(pCtrl_LED->LED.GlobalLed != LEDNOCHANGE) 
				SBOXType[Adapter_ID][ChnlMapToSBOXChnl[bChannelNumber]].LEDColor.GlobalLEDColor= pCtrl_LED->LED.GlobalLed;	
			if(pCtrl_LED->Flash != LEDNOCHANGE) 
				SBOXType[Adapter_ID][ChnlMapToSBOXChnl[bChannelNumber]].Flash.GlobalFlash= pCtrl_LED->Flash;						
			break;
		default :
			return;
	}
	return;
}

U8 CAM_InitSATAEnclosure(U8 Adapter_ID)
{
	U8 i=0,j=0;
	U8 bData=0;
	U32 data=0;
	CAM_Ctrl_LED Ctrl_LED;
	U8 bayNum=0;
	U8 boxNum=0;
	U16 I2cDelay;
	U8 sensorNum=0;
	U8 returnStatus=camFAIL;
	PU8 Buffer;
	U8	LayOutChnlID;
	
	for( i = 0; i < MAX_ATA_CHNL; i++ )
		SBOXType[Adapter_ID][i].BOXConfig = NOSBOX;

	for( i = (Adapter_ID*MAX_ATA_CHNL); i < ((Adapter_ID+1)*MAX_ATA_CHNL); i++ ) {
		HDAccess_StatusLED_Buffer[i][0].UsedFlag = FALSE;
		HDAccess_StatusLED_Buffer[i][1].UsedFlag = FALSE;
		
		HDAccess_ActiveLED_Buffer[i][0].UsedFlag = FALSE;
		HDAccess_ActiveLED_Buffer[i][1].UsedFlag = FALSE;
				
		HDAccess_GlobalLED_Buffer[i][0].UsedFlag = FALSE;
		HDAccess_GlobalLED_Buffer[i][1].UsedFlag = FALSE;
		
		PlugOff_Flag[i] = FALSE;
	}
	
	if (AllHDAccessSetFlag[Adapter_ID] != TRUE) {	/* set default HDActive Leds */
		CAM_SBOX_HDAccess_LEDs[Adapter_ID] = ActiveLED;
		for (i=(Adapter_ID*MAX_ATA_CHNL);i<((Adapter_ID+1)*MAX_ATA_CHNL);i++) {
			CAM_SBOX_HDAccessLED[i].Active_LED.Flash = FASTFLASH;
			CAM_SBOX_HDAccessLED[i].Active_LED.LED_OnOff = LEDON;
		}
	}
	
	AllHDAccessSetFlag[Adapter_ID] = TRUE;	
	AdapterInitFlag[Adapter_ID] = TRUE;
	CAMInitI2CQ(Adapter_ID);

#ifdef Encryption
	if(gAdapterConfig[Adapter_ID].Features & WITH_EncryKEY)
		return returnStatus;
#endif
	
	/* send data to PSS10398 chip, just for its internal inital */
	/* mask ERR & Finish and clear ERR & Finish */
	WritePortWord(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, 0x0F0F);	
	
	/* prepare SBUS buffer */
	Buffer=(PU8)(SerialBusAddr[Adapter_ID].MemPollVirAddr[0]);
	i=0;
	*(Buffer+i)=0x00; /* Address */
	i++;
	*(Buffer+i)=0xC0; /* Control */ /* initial step, don't care ACK */
	i++;
	*(Buffer+i)=0; /* Byte Count */
	
	for (i=0;i<INITI2CTIME;i++) {
		/* Wait for Series bus busy */	
		for (I2cDelay=0;I2cDelay<MAXDELAYFORI2C;I2cDelay++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (bData & 0x20)	/* Busy or ERR */
				CAM_ResetSerialBusMachine(Adapter_ID);
			else
				break;
		}
		
		/* send serial bus master command */
		WritePortUlong(HostBaseAddr[Adapter_ID], oFFSETSBusMasterBaseAddr, (U32)SerialBusAddr[Adapter_ID].MemPollPhyAddress[0]);
		
		/* wait return */
		for (I2cDelay=0;I2cDelay<MAXDELAYFORI2C;I2cDelay++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (bData & 0x0F)	/* FINISH or ERR */
				break;
		}
		CAM_ResetSerialBusMachine(Adapter_ID);
	}

	/* prepare SBUS buffer */
	Buffer=(PU8)(SerialBusAddr[Adapter_ID].MemPollVirAddr[0]);
	i=0;
	*(Buffer+i)=0x00; /* Address */
	i++;
	*(Buffer+i)=0x80; /* Control */
	i++;
	*(Buffer+i)=0; /* Byte Count */
	i++;
	i++;		

	bData = Data2SerialBus(Adapter_ID, (U32)SerialBusAddr[Adapter_ID].MemPollPhyAddress[0]);
		
	if (bData == TRUE)	/* FINISH done,I2C work , */ 
	{
		I2CBusy[Adapter_ID] =TRUE;
		/* read DEVICE ID, check PSS10398 exist? */
		for (i=0;i<MAX_ATA_CHNL;i++) {
			/*bData=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x00+0x10*i)<<1)+1),1);*/
			
			/* read DEVICE ID */

			/* prepare SBUS buffer */
			Buffer=(PU8)(SerialBusAddr[Adapter_ID].MemPollVirAddr[0]);
			j=0;
			*(Buffer+j)=((0x00+0x10*i)<<1)+1; /* bit 0 : R/W 1/0 */ /* Address */
			j++;
			*(Buffer+j)=0x80; /* Control */
			j++;
			*(Buffer+j)=1; /* Byte Count */
			j++;	
			j++;
			*((PU32)(Buffer+j)) = (U32)SerialBusAddr[Adapter_ID].MemPollPhyAddress[1];
			

			bData = Data2SerialBus(Adapter_ID, (U32)SerialBusAddr[Adapter_ID].MemPollPhyAddress[0]);
	
			if (bData != TRUE)	/* ERROR done */
			{
				break;			
			}
			
			data = (*((PU32)(SerialBusAddr[Adapter_ID].MemPollVirAddr[1])));
			/* read data */

			bData = (U8)data;

			/* check box type, and not support mix types */
			if(bData & SBOX_4){	/* 4 bay */
				if (!(bData & SBOX_NO_PWM)) { /* PWM */  /* SS4100 */
					sensorNum = 2;
			
					if((i) && (SBOXType[Adapter_ID][0].BOXType != SuperSwap4100)){
						break;
					}				
				}
				else { /* no PWM */ /* OEM 4-channel */
					/* check bit 2~ bit 4 [0001 1100] to determine box class */
					if((bData & 0x1C) == 0x08) 
						sensorNum = 2;
					else if((bData & 0x1C) == 0x1C)
						sensorNum = 3;
					else /* nonsupport type */
						break;	
			
					if((i) && (SBOXType[Adapter_ID][0].BOXType != PSS10398_4_CHNL)){
						break;
					}	
				}	
			}
			else{	/* 1 bay */
				if (!(bData & SBOX_NO_PWM)) { /* PWM */ /* SS1100 */
					sensorNum = 1;			
					if((i) && (SBOXType[Adapter_ID][0].BOXType != SuperSwap1100)){
						break;					
					}				
				}
				else { /* no PWM */ /* OEM 1-channel */
					sensorNum = 1;			
					if((i) && (SBOXType[Adapter_ID][0].BOXType != PSS10398_1_CHNL)){
						break;					
					}
				}
			}

			
			/* reset I2C LED */
			/* prepare SBUS buffer */
			Buffer=(PU8)(SerialBusAddr[Adapter_ID].MemPollVirAddr[0]);
			j=0;
			*(Buffer+j)=((0x09+0x10*i)<<1); /* Address */
			j++;
			*(Buffer+j)=0x80; /* Control */
			j++;
			*(Buffer+j)=1; /* Byte Count */
			j++;
			j++;		
			*(Buffer+j) = 0x80;

			Data2SerialBus(Adapter_ID, (U32)SerialBusAddr[Adapter_ID].MemPollPhyAddress[0]);
			
			returnStatus=camSUCCESS;
			 
			bData = (U8)data;
			if (bData & SBOX_4) { /* 4 bay */
				bayNum = MAX_BAY_PER_BOX; /* 4 */
				
				for (j=0;j<MAX_BAY_PER_BOX;j++) {
					SBOXType[Adapter_ID][i+j].BOXConfig |= SBOXEXIST | SBOXCOMBO;
					SBOXType[Adapter_ID][i+j].BOXSensorNum = sensorNum;
					
					SBOXType[Adapter_ID][i+j].BOXID = (j<<BayShiftBits) | (boxNum+Adapter_ID*MAX_ATA_CHNL);
					
					LayOutChnlID = CAM_ATA_Chnl_InMAP(Adapter_ID*MAX_ATA_CHNL+i+j);
					/* check PWM */
					if (!(bData & SBOX_NO_PWM)) { /* PWM */ /* SS4100 */
						SBOXType[Adapter_ID][i+j].BOXType = SuperSwap4100;
					#if !defined( SS4100_WITH_PWM )
						/* build map SBOXToCh table */
						gBOXConfig[LayOutChnlID].BOXID=SBOXType[Adapter_ID][i+j].BOXID;
						/*gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i+j].BOXType = SBOXType[Adapter_ID][i+j].BOXType;*/
						gBOXConfig[LayOutChnlID].BOXConfig |= COMBO;
						gBOXConfig[LayOutChnlID].BOXLEDConfig = (WITH_STATUS_LED | WITH_FLASH_OF_STATUS_LED | WITH_TURN_OFF_STATUS_LED | WITH_ACTIVE_LED | WITH_FLASH_OF_ACTIVE_LED | WITH_TURN_OFF_ACTIVE_LED | WITH_Global_LED | WITH_FLASH_OF_Global_LED | WITH_TURN_OFF_Global_LED);
						gBOXConfig[LayOutChnlID].NumofBay = MAX_BAY_PER_BOX;
						
						ChnlMapToSBOXChnl[LayOutChnlID]=i+j;
					
						/* check sensor */
						if (ReadWinBondID_W83782D(Adapter_ID,(U8)(i+j)) == WINBONDW83782DID)
							gBOXConfig[LayOutChnlID].BOXConfig |= WITH_SENSOR;
						else
							gBOXConfig[LayOutChnlID].BOXConfig &= ~WITH_SENSOR;
					#else
						SBOXType[Adapter_ID][i+j].BOXConfig |= SBOXWITHPWM|SBOXWITHPWMUNASSIGNED;
						/* will build map SBOXToCh table when CAM_InitATAChannel */
					#endif
					}
					else { /* no PWM */ /* OEM 4-channel */
						SBOXType[Adapter_ID][i+j].BOXType = PSS10398_4_CHNL;
						/* build map SBOXToCh table */
						gBOXConfig[LayOutChnlID].BOXID=SBOXType[Adapter_ID][i+j].BOXID;
						/*gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i+j].BOXType = SBOXType[Adapter_ID][i+j].BOXType;*/
						gBOXConfig[LayOutChnlID].BOXConfig |= COMBO;
						gBOXConfig[LayOutChnlID].BOXLEDConfig = (WITH_STATUS_LED | WITH_FLASH_OF_STATUS_LED | WITH_TURN_OFF_STATUS_LED | WITH_ACTIVE_LED | WITH_FLASH_OF_ACTIVE_LED | WITH_TURN_OFF_ACTIVE_LED | WITH_Global_LED | WITH_FLASH_OF_Global_LED | WITH_TURN_OFF_Global_LED);
						gBOXConfig[LayOutChnlID].NumofBay = MAX_BAY_PER_BOX;
						
						ChnlMapToSBOXChnl[LayOutChnlID]=i+j;
					
						/* check sensor */
						if (ReadWinBondID_W83782D(Adapter_ID,(U8)(i+j)) == WINBONDW83782DID)
							gBOXConfig[LayOutChnlID].BOXConfig |= WITH_SENSOR;
						else
							gBOXConfig[LayOutChnlID].BOXConfig &= ~WITH_SENSOR;
					}					
				}
			}
			else { /* 1 bay */
				bayNum = 1;
				SBOXType[Adapter_ID][i].BOXConfig |= SBOXEXIST;
				SBOXType[Adapter_ID][i].BOXSensorNum = sensorNum;				
				SBOXType[Adapter_ID][i].BOXID = (0<<BayShiftBits) | (boxNum+Adapter_ID*MAX_ATA_CHNL);
				
				/* check PWM */
				if (!(bData & SBOX_NO_PWM)) { /* PWM */ /* SS1100 */
					SBOXType[Adapter_ID][i].BOXType = SuperSwap1100;
					SBOXType[Adapter_ID][i].BOXConfig |= SBOXWITHPWM|SBOXWITHPWMUNASSIGNED;
				}
				else { /* no PWM */ /* OEM 1-channel */
					SBOXType[Adapter_ID][i].BOXType = PSS10398_1_CHNL;
					/* build map table */
					gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i].BOXID=SBOXType[Adapter_ID][i].BOXID;
					/*gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i].BOXType = SBOXType[Adapter_ID][i].BOXType;*/
					gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i].BOXLEDConfig = (WITH_STATUS_LED | WITH_FLASH_OF_STATUS_LED | WITH_TURN_OFF_STATUS_LED | WITH_ACTIVE_LED | WITH_FLASH_OF_ACTIVE_LED | WITH_TURN_OFF_ACTIVE_LED);
					gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i].NumofBay = 1;
					ChnlMapToSBOXChnl[Adapter_ID*MAX_ATA_CHNL+i]=i;

					/* check sensor */
					if (ReadWinBondID_W83782D(Adapter_ID,i) == WINBONDW83782DID)
						gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i].BOXConfig |= WITH_SENSOR;
					else
						gBOXConfig[Adapter_ID*MAX_ATA_CHNL+i].BOXConfig &= ~WITH_SENSOR;
				}	
			}
			
			for (j=i;j<(i+bayNum);j++) {
				/* get defult status LED */
				bData = (U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x04+0x10*j)<<1)+1),1);
				bData = (U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x04+0x10*j)<<1)+1),1);				
				switch(bData & 0x03){
					case 1:
						DefaultStatusLed = LED_Red;
						break;
					case 2:
						DefaultStatusLed = LED_Green;
						break;
					case 3:	
						DefaultStatusLed = LED_Amber;
						break;
					default:
						DefaultStatusLed = LED_Amber;
						break;
				}

				SBOXType[Adapter_ID][j].LEDColor.StatusLEDColor = DefaultStatusLed;

				/* set Global LED */
				if (CAM_SBOX_HDAccess_LEDs[Adapter_ID] & GlobalLED) {
					if (CAM_SBOX_HDAccessLED[Adapter_ID*MAX_ATA_CHNL+j].Global_LED.LED_OnOff==LEDOFF) {
						Ctrl_LED.LED.GlobalLed=LEDOFF;
						Ctrl_LED.Flash=CAM_SBOX_HDAccessLED[Adapter_ID*MAX_ATA_CHNL+j].Global_LED.Flash;
					}
					else { /* HDAccess LEDON */
						Ctrl_LED.LED.GlobalLed=LEDNOCHANGE;
						Ctrl_LED.Flash=NOFLASH;
					}
				}
				else {
					Ctrl_LED.LED.GlobalLed=LEDOFF;
					Ctrl_LED.Flash=NOFLASH;
				}
				SBOXType[Adapter_ID][j].Flash.GlobalFlash= NOFLASH;
				
				HDAccess_GlobalLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].Buffer_LED.LED.GlobalLed=Ctrl_LED.LED.GlobalLed;
				HDAccess_GlobalLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].Buffer_LED.Flash=Ctrl_LED.Flash;
				HDAccess_GlobalLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].NeedRetryFlag = FALSE;
		
				CAM_SBOX_GlobalLED(Adapter_ID,j, (PCAM_SBOX_LED)&Ctrl_LED,InternalI2CReturn);
				
				/* set Status LED */
				if (CAM_SBOX_HDAccess_LEDs[Adapter_ID] & StatusLED) {
					if (CAM_SBOX_HDAccessLED[Adapter_ID*MAX_ATA_CHNL+j].Status_LED.LED_OnOff==LEDOFF) {
						Ctrl_LED.LED.StatusLed=LEDOFF;
						Ctrl_LED.Flash=CAM_SBOX_HDAccessLED[Adapter_ID*MAX_ATA_CHNL+j].Status_LED.Flash;
					}
					else { /* HDAccess LEDON */
						Ctrl_LED.LED.StatusLed=SBOXType[Adapter_ID][j].LEDColor.StatusLEDColor;
						Ctrl_LED.Flash=NOFLASH;
					}
				}
				else {
					Ctrl_LED.LED.StatusLed=LEDNOCHANGE;
					Ctrl_LED.Flash=LEDNOCHANGE;
				}
				
				HDAccess_StatusLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].Buffer_LED.LED.StatusLed=Ctrl_LED.LED.StatusLed;
				HDAccess_StatusLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].Buffer_LED.Flash=Ctrl_LED.Flash;
				HDAccess_StatusLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].NeedRetryFlag = FALSE;
				
				Ctrl_LED.LED_Type=StatusLED;
				CAM_SBOX_StatusLED(Adapter_ID,j, (PCAM_SBOX_LED)&Ctrl_LED,InternalI2CReturn);
				
				/* set Active LED */
				if (CAM_SBOX_HDAccess_LEDs[Adapter_ID] & ActiveLED) {
					if (CAM_SBOX_HDAccessLED[Adapter_ID*MAX_ATA_CHNL+j].Active_LED.LED_OnOff==LEDOFF) {
						Ctrl_LED.LED.ActiveLed=LEDOFF;
						Ctrl_LED.Flash=CAM_SBOX_HDAccessLED[Adapter_ID*MAX_ATA_CHNL+j].Active_LED.Flash;
					}
					else { /* HDAccess LEDON */
						Ctrl_LED.LED.ActiveLed=LEDOFF;
						Ctrl_LED.Flash=NOFLASH;
					}
				}
				else {
					Ctrl_LED.LED.ActiveLed=LEDOFF;
					Ctrl_LED.Flash=LEDNOCHANGE;
				}
				
				HDAccess_ActiveLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].Buffer_LED.LED.ActiveLed=Ctrl_LED.LED.ActiveLed;
				HDAccess_ActiveLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].Buffer_LED.Flash=Ctrl_LED.Flash;				
				HDAccess_ActiveLED_Buffer[Adapter_ID*MAX_ATA_CHNL+j][0].NeedRetryFlag = FALSE;
				
				Ctrl_LED.LED_Type=ActiveLED;
				CAM_SBOX_ActiveLED(Adapter_ID,j, (PCAM_SBOX_LED)&Ctrl_LED,InternalI2CReturn);
				
				
				SBOXType[Adapter_ID][j].LEDColor.GlobalLEDColor = LEDOFF; /* default global color */
				
			} 
			for (j=0;j<(bayNum-1);j++) 
				i++;
			boxNum++;

		}/* end for (i) */
		
		/* When error, the next I2C comand will error of this version of 399, so add a I2C command. */
		/* mask ERR & Finish and clear ERR & Finish */
		CAM_ResetSerialBusMachine(Adapter_ID);

		I2CBusy[Adapter_ID] =FALSE;		
		CAM_StartI2C(Adapter_ID);
	} /* end if (bData & 0x02) */

	return (returnStatus);
}

void CAM_InitWithoutDiskSATAEnclosures(U8 Adapter_ID,PCAM_Ctrl_LED pCtrl_LED)
{
	U8 i;
	
	if (!(SBOXType[Adapter_ID][0].BOXConfig & SBOXEXIST))
		return; /* no enclosure */
	
	if (SBOXType[Adapter_ID][0].BOXConfig & SBOXWITHPWM) {
		for (i=0;i<MAX_ATA_CHNL;i++) {
			if (!(SBOXType[Adapter_ID][i].BOXConfig & SBOXEXIST))
				break;
			/* SBOX not assigned */	
			if (SBOXType[Adapter_ID][i].BOXConfig & SBOXWITHPWMUNASSIGNED) {
				switch (pCtrl_LED->LED_Type) {
					case ActiveLED:
						CAM_SBOX_ActiveLED(Adapter_ID,i, (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
						break;
					case StatusLED:
						CAM_SBOX_StatusLED(Adapter_ID,i, (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
						if(pCtrl_LED->LED.StatusLed != LEDNOCHANGE)
							SBOXType[Adapter_ID][i].LEDColor.StatusLEDColor= pCtrl_LED->LED.StatusLed;
						break;
					case GlobalLED:
						CAM_SBOX_GlobalLED(Adapter_ID,i, (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
						if(pCtrl_LED->LED.GlobalLed != LEDNOCHANGE) 
							SBOXType[Adapter_ID][i].LEDColor.GlobalLEDColor= pCtrl_LED->LED.GlobalLed;	
						if(pCtrl_LED->Flash != LEDNOCHANGE) 
							SBOXType[Adapter_ID][i].Flash.GlobalFlash= pCtrl_LED->Flash;						
						break;
					default :
						return;
				}				
			}
		}
	}
	else { /* no PWM */
		for (i=0;i<MAX_ATA_CHNL;i++) {
			/* disk not exist bot sbox exist */
			if (!(gDrvConfig[Adapter_ID*MAX_ATA_CHNL+i][0].DevFlag & DEV_EXIST) || (gChnlConfig[Adapter_ID*MAX_ATA_CHNL+i].ChnlType != CHNL_SATA )) {
				U8	BoxId=CAM_ATA_Chnl_OutMAP(i); 
				if (SBOXType[Adapter_ID][BoxId].BOXConfig & SBOXEXIST) {
					switch (pCtrl_LED->LED_Type) {
						case ActiveLED:
							CAM_SBOX_ActiveLED(Adapter_ID,BoxId, (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
							break;
						case StatusLED:
							CAM_SBOX_StatusLED(Adapter_ID,BoxId, (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
							if(pCtrl_LED->LED.StatusLed != LEDNOCHANGE)
								SBOXType[Adapter_ID][BoxId].LEDColor.StatusLEDColor= pCtrl_LED->LED.StatusLed;
							break;
						case GlobalLED:
							CAM_SBOX_GlobalLED(Adapter_ID,BoxId, (PCAM_SBOX_LED)pCtrl_LED,InternalI2CReturn);
							if(pCtrl_LED->LED.GlobalLed != LEDNOCHANGE) 
								SBOXType[Adapter_ID][BoxId].LEDColor.GlobalLEDColor= pCtrl_LED->LED.GlobalLed;	
							if(pCtrl_LED->Flash != LEDNOCHANGE) 
								SBOXType[Adapter_ID][BoxId].Flash.GlobalFlash= pCtrl_LED->Flash;						
							break;
						default :
							return;
					}
				}
				else
					break;
			}
		}
	}

	return;
	
}

void CAM_CloseSATAEnclosure(U8 Adapter_ID)
{
	U8 i;
	U16 j;
	U8 bData=0;
	PU8 Buffer;
		
	for (i=0;i<MAX_ATA_CHNL;i++) {
		if (SBOXType[Adapter_ID][i].BOXConfig & SBOXEXIST)
			break;
	}
	
	if (i >= MAX_ATA_CHNL)
		return;

	/* mask ERR & Finish and  */
	WritePortWord(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, 0x0F00);	
		
	if(I2CBusy[Adapter_ID] != TRUE)
		I2CBusy[Adapter_ID] =TRUE;
	else{
		for (j=0;j<MAXDELAYFORI2C;j++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (bData & 0x0F)	/* FINISH or ERR */
			{
				CAM_ResetSerialBusMachine(Adapter_ID);
				break;
			}			
		}
	}

	/* reset chip of Enclosure */
	/* reset 10398 chip */
	for (i=0;i<MAX_ATA_CHNL;i++) { /* form box 3, 2, 1 then 0 */ 
		if (SBOXType[Adapter_ID][MAX_ATA_CHNL-i-1].BOXConfig & SBOXEXIST) {
			/* mask ERR & Finish and clear ERR & Finish */
			WritePortWord(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, 0x0F0F);	

			/* reset I2C LED */
			/* prepare SBUS buffer */
			Buffer=(PU8)(SerialBusAddr[Adapter_ID].MemPollVirAddr[0]);
			j=0;
			*(Buffer+j)=((0x09+0x10*(MAX_ATA_CHNL-i-1))<<1); /* Address */
			j++;
			*(Buffer+j)=0x80; /* Control */
			j++;
			*(Buffer+j)=1; /* Byte Count */
			j++;
			j++;		
			*(Buffer+j) = 0x80;

			Data2SerialBus(Adapter_ID, (U32)SerialBusAddr[Adapter_ID].MemPollPhyAddress[0]);			
		}
	}

	for( i = 0; i < MAX_ATA_CHNL; i++ )
		SBOXType[Adapter_ID][i].BOXConfig = NOSBOX;	

	/* clear all I2C datas in CAM_I2CQueue */
	I2CQueueHead[Adapter_ID] = I2CQueueTail[Adapter_ID];
		
	return;
}

/* setup fan divisor to 010b, which gives a min RPM = 1.35e06/(255 * 2^2) = 1323
 * set fan1, fan2, fan3 divisor
 * [fan1] bit2 = Reg: 5D (bit5), bit1 = Reg: 47 (bit5), bit0 = Reg: 47 (bit4)
 * [fan2] bit2 = Reg: 5D (bit6), bit1 = Reg: 47 (bit7), bit0 = Reg: 47 (bit6)
 * [fan3] bit2 = Reg: 5D (bit7), bit1 = Reg: 4B (bit7), bit0 = Reg: 4B (bit6)
 */
void SBoxSetupAllFanDivisor(U8 Adapter_ID, U8 bID, U8 Divisor)
{
	U8 fd;
	U32 com_data=0;
	
	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);

	/* read divisor bit 2 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5D50,2);
	fd=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd &= ~0xE0;	/* fan1~3 bit2 of divisor are bit 5~7 of reg5D */
	fd |= (Divisor & 0x04 ) << 3;	/* set fan1 bit2 of divisor */
	fd |= (Divisor & 0x04 ) << 4;	/* set fan2 bit2 of divisor */
	fd |= (Divisor & 0x04 ) << 5;	/* set fan3 bit2 of divisor */

	com_data = ((U32)fd) << 16;
	com_data |= 0x5D50;  
	/* write divisor bit 2 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,com_data,3);

	/* read divisor bit 1~0 of fan1, fan2 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4750,2);
	fd=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd &= ~0xF0;	/* fan1~2 bit1,0 of divisor is bit 4~7 of reg47 */
	
	fd |= (Divisor & 0x03 ) << 4;	/* fan1 bit1,0 of divisor is bit 4, 5 of reg47 */
	fd |= (Divisor & 0x03 ) << 6;	/* fan2 bit1,0 of divisor is bit 6, 7 of reg47 */

	com_data = ((U32)fd) << 16;
	com_data |= 0x4750;  	
	/* write divisor bit 1~0 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,com_data,3);

	/* read divisor bit 1~0 of fan3 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4B50,2);
	fd=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd &= ~0xC0;	/* fan3 bit1,0 of divisor is bit 6,7 of reg4B */
	
	fd |= (Divisor & 0x03 ) << 6;	/* fan3 bit1,0 of divisor is bit 4, 5 of reg4B */

	com_data = ((U32)fd) << 16;
	com_data |= 0x4B50;  	
	/* write divisor bit 1~0 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,com_data,3);

	
	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);
	return;
}

U32 SBoxReadFanStatus(U8 Adapter_ID, U8 bID)
{
    U8	fd, fd2;	/* FAN1DIV */
    U32	divisor;
    U8	fan1cnt;	/* FAN1COUNT */
    const U32 Dividend = 0x149970; /* 1.35 x 10^6 */
	U32	rpm=0;


	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);

	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5D50,2);
	fd=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd &= 0x20;		/* get bit5 - FAN1DIV bit2 */
	fd = fd >> 3;	/* move to bit2 */


	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4750,2);
	fd2=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd2 &= 0x30;	/* get bit5,4 - FAN1DIV bit1,0 */
	fd2 = fd2 >> 4;	/* move to bit1,0 */

	fd |= fd2;
	divisor = 1 << fd;


	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2850,2);
	fan1cnt=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	if (!fan1cnt * divisor)
		rpm = 0;
	else   
		rpm = Dividend / (fan1cnt * divisor);

	return rpm;
}

/* read Temp from WinBond, in deg C */
/*   this is temp1 of the IC */

U16 SBoxReadTemperature(U8 Adapter_ID, U8 bID)
{
	U8  temp, mag;
	int sign;

	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);
	
	/*read Temperature reading byte (27h)*/
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2750,2);
	temp=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	if(temp & 0x80){ /* negative */
		sign = 1;
		mag = ~temp;  
		mag &= ~0x80;
		mag += 1;
		temp = mag * sign;
	}
	return (U16)temp*2;
}

/* Read 5V Voltage Level. The LSB is 0.016 V=16 mV
 * this gives 8 bit = 4.096 V max voltage. The serial
 * resistor is used to drop the 5V level to 4.096V
*/
U32 SBoxReadVoltage5V(U8 Adapter_ID, U8 bID)
{
    U8 vbase=16; /* 8-bit ADC has a 16mv LSB */
    U32 voltage;
    U8 regval;
    U8 SR1=34, SR2=50;	/* Vin is divided into R2/(R1+R2), to fit */
                          	/* with 0-4096. SR=Serial Resistor	*/

	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);
	
	/*read +5VIN reading byte */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2350,2);
	regval=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

    voltage = regval * vbase;
    voltage =  voltage  * (SR1 + SR2) / SR2;

   return voltage;
}

U32 SBoxReadVoltage12V(U8 Adapter_ID, U8 bID)
{
    U8 vbase=16; /* 8-bit ADC has a 16mv LSB */
    U32 voltage;
    U8 regval;
    U8 R1=28, R2=10;		/* Vin is divided into R2/(R1+R2), to fit */
							/* with 0-4096 */


	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);
	
	/*read +12VIN reading byte */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2450,2);
	regval=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

    voltage = regval * vbase;
    voltage = voltage * (R1 + R2)/R2;

    return voltage;
}


U32 SBoxReadVoltage3V(U8 Adapter_ID, U8 bID)
{
    U8 vbase=16; /* 8-bit ADC has a 16mv LSB */
    U32 voltage;
    U8 regval;


	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);
	
	/*read +12VIN reading byte */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2250,2);
	regval=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	voltage = regval * vbase;

	return voltage;
}

/* this is done by setting bit2 = Reg 5D, bit6
 *                          bit1 = Reg 47, bit7
 *                          bit0 = Reg 47, bit6
 *   for SW4100 only
 */
U32 SBoxReadFan2Status(U8 Adapter_ID, U8 bID)
{
    U8	fd, fd2;	/* FAN2DIV */
    U32	divisor;
    U8	fan2cnt;	/* FAN2COUNT */
    const U32 Dividend = 0x149970; /* 1.35 x 10^6 */
	U32	rpm=0;


	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);

	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5D50,2);
	fd=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd &= 0x40;		/* get bit6 - FAN2DIV bit2 */
	fd = fd >> 4;	/* move to bit2 */


	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4750,2);
	fd2=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd2 &= 0xC0;	/* get bit7,6 - FAN2DIV bit1,0 */
	fd2 = fd2 >> 6;	/* move to bit1,0 */

	fd |= fd2;
	divisor = 1 << fd;


	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2950,2);
	fan2cnt=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	if (!fan2cnt * divisor)
		rpm = 0;
	else   
		rpm = Dividend / (fan2cnt * divisor);

	return rpm;
}

/* this is done by setting bit2 = Reg 5D, bit7
 *                          bit1 = Reg 4B, bit7
 *                          bit0 = Reg 4B, bit6
 *   for SW4100 only
 */
U32 SBoxReadFan3Status(U8 Adapter_ID, U8 bID)
{
    U8	fd, fd2;	/* FAN3DIV */
    U32	divisor;
    U8	fan3cnt;	/* FAN3COUNT */
    const U32 Dividend = 0x149970; /* 1.35 x 10^6 */
	U32	rpm=0;


	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);

	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5D50,2);
	fd=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd &= 0x80;		/* get bit7 - FAN3DIV bit2 */
	fd = fd >> 5;	/* move to bit2 */


	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x4B50,2);
	fd2=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);
	fd2 &= 0xC0;	/* get bit7,6 - FAN1DIV bit1,0 */
	fd2 = fd2 >> 6;	/* move to bit1,0 */

	fd |= fd2;
	divisor = 1 << fd;


	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x2A50,2);
	fan3cnt=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	if (!fan3cnt * divisor)
		rpm = 0;
	else   
		rpm = Dividend / (fan3cnt * divisor);

	return rpm;
}

/* read Temp from WinBond, in deg C */
/*   this is temp2 of the IC 
 *   for SW4100 only
*/

U16 SBoxReadTemperature2(U8 Adapter_ID, U8 bID)
{
	U8  msb, lsb;
	U16 temp, mag;
	int sign;

	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);


	/* select Bank 1 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x014E50,3);

	/* read MSB */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5050,2);
	msb=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* read LSB */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5150,2);
	lsb=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);



	/* restore to bank 0 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x004E50,3);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	temp = ((U16)msb << 1) | ((U16)lsb >> 7);
	/* this value is 9 bit 2's complement */
	/* resolutin is 0.5 deg, range is -128 to 127.5 */
	sign = (temp & 0x100) ? -1 : 1;
	mag = (temp + 1) & 0x1ff;
	temp = mag * sign;
	return temp;
}

/* read Temp from WinBond, in deg C */
/*   this is temp3 of the IC 
 *   for SW4100 only
*/

U16 SBoxReadTemperature3(U8 Adapter_ID, U8 bID)
{
	U8  msb, lsb;
	U16 temp, mag;
	int sign;

	/* set LKSB high register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x01+0x10*bID)<<1),1);


	/* select Bank 1 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x024E50,3);

	/* read MSB */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5050,2);
	msb=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);

	/* read LSB */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x5150,2);
	lsb=(U8)CAM_SerialBusReadDATAPolling(Adapter_ID,0x51,1);



	/* restore to bank 0 */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,0x004E50,3);

	/* set LKSB low register */
	CAM_SerialBusWriteDATAPolling(Adapter_ID,((0x02+0x10*bID)<<1),1);

	temp = ((U16)msb << 1) | ((U16)lsb >> 7);
	/* this value is 9 bit 2's complement */
	/* resolutin is 0.5 deg, range is -128 to 127.5 */
	sign = (temp & 0x100) ? -1 : 1;
	mag = (temp + 1) & 0x1ff;
	temp = mag * sign;
	return temp;
}

U8 Get_SataEnclosure_Datas(U8	bChannelNumber)
{
	PENCLOSURE_TYPE pBox = &gBOXConfig[bChannelNumber];
	PENCLOSURE_STATUS pStatus = &gBOXStatus[bChannelNumber];	
	U8	bData;
	U8	Adapter_ID = bChannelNumber/MAX_ATA_CHNL;	
	U8	bID;
	U16	i;


	if (I2CBusy[Adapter_ID] == TRUE) {
		/* mask ERR & Finish and  */
		WritePortWord(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, 0x0F00);	
			
		for (i=0;i<MAXDELAYFORI2C;i++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (bData & 0x0F)	/* FINISH or ERR */
			{
				CAM_ResetSerialBusMachine(Adapter_ID);
				break;
			}			
		}
		/* call callback instead ISR */
		I2CQueueCurrent[Adapter_ID].callback(&I2CQueueCurrent[Adapter_ID]);
	}
	
	I2CBusy[Adapter_ID] = TRUE;
	bID=ChnlMapToSBOXChnl[bChannelNumber];
			
	if(PlugOff_Flag[bChannelNumber] == FALSE){
		bData = (U8)CAM_SerialBusReadDATAPolling(Adapter_ID,(U8)(((0x09+0x10*bID)<<1)+1),1);	
		pStatus->PowerState = ( bData & bit(6) )? PowerON : PowerOFF;
	}
	else
		pStatus->PowerState = PowerOFF;

	/* only when PowerOn of box, FAN RPM and temp is available*/
	if (pStatus->PowerState == PowerON){
		pStatus->FANStatus = SBoxReadFanStatus(Adapter_ID, bID);
		pStatus->TEMPStatus = SBoxReadTemperature(Adapter_ID, bID);	
		pStatus->VOL5V = SBoxReadVoltage5V(Adapter_ID, bID);	
		pStatus->VOL12V = SBoxReadVoltage12V(Adapter_ID, bID);
		pStatus->VOL3V = SBoxReadVoltage3V(Adapter_ID, bID);
		if((pBox->BOXType == SuperSwap4100) || (pBox->BOXType == PSS10398_4_CHNL) ){
			if(pBox->NumofFAN == 3){
				pStatus->FAN2Status = SBoxReadFan2Status(Adapter_ID, bID);
				pStatus->FAN3Status = SBoxReadFan3Status(Adapter_ID, bID);				
				pStatus->TEMP2Status = SBoxReadTemperature2(Adapter_ID, bID);	
				pStatus->TEMP3Status = SBoxReadTemperature3(Adapter_ID, bID);			
			}
			else{ /* fan and sensor == 2 */
				pStatus->FAN2Status = SBoxReadFan2Status(Adapter_ID, bID);
				pStatus->FAN3Status = 0;							
				pStatus->TEMP2Status = SBoxReadTemperature2(Adapter_ID, bID);	
				pStatus->TEMP3Status = 0;			
			}	
		}
		else{
			pStatus->FAN2Status = 0;
			pStatus->FAN3Status = 0;
			pStatus->TEMP2Status = 0;
			pStatus->TEMP3Status = 0;
		}		
	}
	else{
		pStatus->FANStatus = 0;
		pStatus->TEMPStatus = 0;
		pStatus->VOL5V = 0;
		pStatus->VOL12V = 0;
		pStatus->VOL3V = 0;
	}
		
	pStatus->RemPeriod = pBox->ChkPeriod;
	
	I2CBusy[Adapter_ID] = FALSE;
	CAM_StartI2C(Adapter_ID);
	return TRUE;
}



void Get_SataBox_Status(PCAM_I2C_QUEUE pI2C_QUEUE)
{
	U8	bChannelNumber = pI2C_QUEUE->b_ID;
	U8	Adapter_ID = bChannelNumber/MAX_ATA_CHNL;

	PENCLOSURE_TYPE pBox = &gBOXConfig[bChannelNumber];
	PENCLOSURE_STATUS pStatus = &gBOXStatus[bChannelNumber];
	
	
	pStatus->NeedCheck = FALSE;
		
	/* Need to get box status ? */
	if ( pBox->BOXConfig & WITH_SENSOR) {
		if ( ( pStatus->RemPeriod == 0 )
	  	|| ((pStatus->RemPeriod = pStatus->RemPeriod-10) == 0) ){ 
	 		I2CBusy[Adapter_ID] = FALSE;
	 		Get_SataEnclosure_Datas(bChannelNumber);
		}
		else{
			I2CBusy[Adapter_ID] = FALSE;
			CAM_StartI2C(Adapter_ID);
		}
	}
	else{
		I2CBusy[Adapter_ID] = FALSE;
		CAM_StartI2C(Adapter_ID);
	}
	BOX_Timeout_TimerID[bChannelNumber] = camTimer(Adapter_ID,CAMTimeoutTimes[Adapter_ID].BOX_Check, Chk_BOX_Function[bChannelNumber]);
	return;
}

U8 CAM_SerialBus(U8 Adapter_ID, PCAM_SBUS_TASK pSBTask)
{
	return TRUE;
}


U8 Data2SerialBus(U8 Adapter_ID, U32 WriteData)
{
	U16	I2cDelay;
	U8	bData=0, I2cRetryTime;
	U8	Is_OK=FALSE; 

	for(I2cRetryTime=0; I2cRetryTime<5; I2cRetryTime++){

		/* Wait for Series bus busy */	
		for (I2cDelay=0;I2cDelay<MAXDELAYFORI2C;I2cDelay++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (!(bData & 0x70))	/* can submit task */
				break;
			CAM_ResetSerialBusMachine(Adapter_ID);
		}
		if(I2cDelay >= MAXDELAYFORI2C)
			break;
			
		/* send serial bus master command */
		WritePortUlong(HostBaseAddr[Adapter_ID], oFFSETSBusMasterBaseAddr, (U32)WriteData);
			
		/* wait return */
		for (I2cDelay=0;I2cDelay<MAXDELAYFORI2C;I2cDelay++){
			ReadPortUchar(HostBaseAddr[Adapter_ID], oFFSETSBusControlStatus, bData );
			if (bData & 0x0F){	/* FINISH or ERR */
				CAM_ResetSerialBusMachine(Adapter_ID);
				break;
			}
		}
		if(bData & 0x01){	/* Finish */				
			Is_OK = TRUE;
			break;
		}
	}
	return Is_OK;
}

