/* Copyright (c) 2003 - 2005 Promise Technology, Inc.  All rights reserved.
 *
 * ulsata2.h - defines data structures, constants and scsi template
 *
 */
#ifndef _UL_MOD_H
#define _UL_MOD_H

#include <linux/version.h>

#define PDCNAME "ulsata2"
#define PRODUCT "SATA-II 150/300"
#define VERSION	"1.00.0.14"

#define MAXADAPTER  0x2

#define ChPAda      0x4
#define DrvPCh      0x2
#define MAXCHANNEL  MAXADAPTER*ChPAda
#define MAXDRIVE    MAXADAPTER*ChPAda*DrvPCh
#define MAXTIMER    MAXADAPTER*ChPAda*2

#define QLENGTH     64
#define MAXSGTABLE  32

/* Octopus-2 */
#define DID575      0x3D75
#define DID579      0x3574
#define DID518      0x3D18
#define DID775      0x3D73
#define DID779      0x3577
#define DID718      0x3D17


unsigned short UL_DEVID[7] = { DID575, DID518, DID579, DID775, DID718, DID779,0};

#define Q_INITIAL   0x1
#define Q_STARTED   0x2
#define Q_POSTED    0x4
#define Q_PENDING   0x8
#define Q_COMPLETED 0x10
#define Q_FINISHED  0x20
#define Q_START_MASK (Q_STARTED | Q_POSTED | Q_PENDING | Q_COMPLETED | Q_FINISHED)

#define LOGATA      0x1
#define LOGATAPI    0x2

/* char device ioctl commands */
#define STIO_GET_ADAPTERCOUNT	0x00004001
#define STIO_GET_ADAPTERTYPE	0x00004002
#define STIO_GET_ADAPTERINFO	0x00004003
#define STIO_FIND_FLASHMEM	0x00004004
#define STIO_ERASE_FLASHMEM	0x00004005
#define STIO_READ_FLASHMEM	0x00004006
#define STIO_WRITE_FLASHMEM	0x00004007

/* define S.M.A.R.T scsi commands */
#define SCSI_TEST_UNIT_READY		0x00
#define SCSI_REQUEST_SENSE		0x03
#define SCSI_INQUIRY			0x12
#define SCSI_MODE_SELECT		0x15
#define SCSI_MODE_SENSE			0x1a
#define SCSI_RECEIVE_DIAGNOSTIC		0x1c
#define SCSI_SEND_DIAGNOSTIC		0x1d
#define SCSI_LOG_SENSE			0x4d
#define SCSI_MODE_SELECT_10		0x55
#define SCSI_MODE_SENSE_10		0x5a

typedef struct _ulsata2_adapter ulsata2_adapter_t;
typedef struct _ulsata2_channel ulsata2_channel_t;
typedef struct _ulsata2_drive ulsata2_drive_t;
typedef struct _ulsata2_timer ulsata2_timer_t;
typedef struct _ulsata2_req ulsata2_req_t;
typedef struct _ulsata2_info ulsata2_info_t;

struct _ulsata2_adapter {
	unsigned char 	id;				/* adapter #: 0 ~ 3 */
	unsigned char	present;			/* 1: present */
	unsigned short 	deviceid;			/* adapter deviceid */
	unsigned int 	irq;				/* interrupt */
	unsigned char	asic_rev;
	unsigned int	base[6];			/* base address */
	unsigned int	range[6];			/* base address range */
	unsigned int 	rombase;
	void *		camVirMem;			/* for NCQ only */
	ADAPTER_CONFIG_INFO	*cam_ada;		/* cam adapter info */
	CAM_INFO	*cam_info;			/* cam info */
	spinlock_t	ulsata2_lock;			/* spin lock */
	ulsata2_channel_t	*pchannel[ChPAda];

	/* linux only */
	struct pci_dev	*pci_dev;			/* pci structure */
	struct Scsi_Host	*host;			/* scsi host */
} __attribute__ ((packed));

struct _ulsata2_channel {
	ulsata2_adapter_t	*padapter;
	unsigned char		id;			/* channel id */
	CHNL_CONFIG		*cam_chn;		/* cam channel info */
	ulsata2_drive_t		*pdrive[DrvPCh];
} __attribute__ ((packed));

struct _ulsata2_drive {
	ulsata2_channel_t	*pchannel;
	/* CAM_IDENTIFY_DATA	identify;*/		/* hd identify */
	DEV_CONFIG		*cam_dev;		/* cam device info */
	unsigned char		retries;
} __attribute__ ((packed));

struct _ulsata2_timer {					/* timer table */
	struct timer_list timer;			/* timer list */
	int id;
	unsigned char status;				/* timer status */
	void (*callback)(void);				/* my timer callback */
	struct _ulsata2_timer *next;
} __attribute__ ((packed));

struct _ulsata2_req {
	ATA_TASK cam_task;				/* submit to cam */
	ATA_TASK cam_task2;				/* backup cam_task */
	CAMSG  	 cam_sg[MAXSGTABLE];			/* cam sg table */
	void	 *cmd;					/* for SCpnt */
	unsigned char *buffer;				/* for atapi */
	int	 buffer_size;				/* for atapi */
	int	 stat;
	int	 id;
	struct _ulsata2_req *next;
} __attribute__ ((packed));

struct _ulsata2_info {					/* proc info */
	char	*buffer;
	int	length;
	int	offset;
	int	position;
} __attribute__ ((packed));

/* structure for char ioctl args */
typedef enum _hardwaretype {
	SATA150 = 0,			/* SATAII 150 series */
	SATA518,			/* SATAII 150 TX4 */
	SATA575,			/* SATAII 150 TX2plus */
	SATA579,				/* SATAII 579 */
	SATA718,			/* SATA 300 TX4 */
	SATA775,			/* SATA 300 TX2plus */
	SATA779				/* SATA 300 779 */
} hardwaretype;

typedef struct _tag_adaptercount {
	unsigned char adaptercount;	/* total adapter count */
} tag_adaptercount_t;

typedef struct _tag_adaptertype {
	unsigned char adapter;		/* adapter index */
	hardwaretype adaptertype;	/* adapter type */
} tag_adaptertype_t;

typedef struct _tag_adapterinfo {
	unsigned char adapter;		/* adapter index */
	unsigned short deviceid;	/* adapter device id */
} tag_adapterinfo_t;

/* flash operation */
typedef struct _tag_flash {
	unsigned char adapter;		/* adapter index */
	unsigned long offset;		/* mem offset */
	unsigned long buffer_size;	/* buffer size of buffer */
	unsigned char *buffer;		/* data buffer */
} tag_flash_t;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,1)
#define ulsata2 {						\
	next: NULL,						\
	proc_info: ulsata2_proc_info,				\
	name: PDCNAME,						\
	detect: ulsata2_detect,					\
	release: ulsata2_release,				\
	ioctl: ulsata2_ioctl,					\
	info: NULL,						\
	command: NULL,						\
	queuecommand: ulsata2_queuecommand,			\
	abort: ulsata2_scsi_abort,				\
	reset: ulsata2_scsi_reset,				\
	eh_abort_handler: ulsata2_scsi_eh_abort,			\
	eh_device_reset_handler: ulsata2_scsi_eh_device_reset,	\
	eh_host_reset_handler: ulsata2_scsi_eh_host_reset,	\
	slave_attach: NULL,					\
	bios_param: ulsata2_bios_param,				\
	can_queue: QLENGTH,					\
	this_id: -1,						\
	sg_tablesize: MAXSGTABLE,				\
	cmd_per_lun: 1,						\
	present: 0,						\
	unchecked_isa_dma: 0,					\
	use_clustering: DISABLE_CLUSTERING,			\
	use_new_eh_code: 1,					\
	emulated: 1						\
};
#else
#define ulsata2 {
	.proc_name	= "ulsata2",
	.proc_info	= ulsata2_proc_info,
	.name		= "Promise SATA-II series controller",
	.detect		= ulsata2_detect,
	.release	= ulsata2_release,
	.queuecommand	= ulsata2_queuecommand,
	.eh_abort_handler	= ulsata2_scsi_eh_abort,
	.eh_host_reset_handler	= ulsata2_scsi_eh_host_reset,
	.bios_param	= ulsata2_bios_param,
	.can_queue	= QLENGTH,
	.this_id	= -1,
	.sg_tablesize	= MAXSGTABLE,
	.cmd_per_lun	= 1,
	.use_clustering	= DISABLE_CLUSTERING,
	.emulated: 1,
	.highmem_io	= 1
};
#endif

#endif /* end of _UL_MOD_H */
